@@ -2,6 +2,7 @@
 
 using System;
 using MonoDevelop.Projects;
+using MonoDevelop.Ide.Gui;
 
 namespace MonoDevelop.Autotools
 {
@@ -10,14 +11,14 @@ namespace MonoDevelop.Autotools
 	public partial class GenerateMakefilesDialog : Gtk.Dialog
 	{
 
-		public GenerateMakefilesDialog (Combine combine)
+		public GenerateMakefilesDialog (Solution solution)
 		{
 			this.Build();
 
-			for (int i = 0; i < combine.Configurations.Count; i ++) {
-				CombineConfiguration cc = (CombineConfiguration) combine.Configurations [i];
-				comboConfigs.AppendText (cc.Name);
-				if (cc.Name == combine.ActiveConfiguration.Name)
+			for (int i = 0; i < solution.Configurations.Count; i ++) {
+				SolutionConfiguration cc = (SolutionConfiguration) solution.Configurations [i];
+				comboConfigs.AppendText (cc.Id);
+				if (cc.Id == IdeApp.Workspace.ActiveConfiguration)
 					comboConfigs.Active = i;
 			}
 		}
