@@ -5,6 +5,7 @@ using MonoDevelop.Projects;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Pads;
 using MonoDevelop.DesignerSupport.PropertyGrid;
+using MonoDevelop.Ide.Gui.Components;
 
 namespace MonoDevelop.DesignerSupport.Projects
 {
@@ -41,11 +42,10 @@ namespace MonoDevelop.DesignerSupport.Projects
 			// Don't use the CurrentNode property here since it may not be properly initialized when the event is fired.
 			ITreeNavigator nav = Tree.GetNodeAtObject (obj);
 			if (nav != null) {
-				CombineEntry ce = (CombineEntry) nav.GetParentDataItem (typeof(CombineEntry), true);
+				IWorkspaceFileObject ce = (IWorkspaceFileObject) nav.GetParentDataItem (typeof(IWorkspaceFileObject), true);
 				if (ce != null) {
-					using (IProgressMonitor mon = IdeApp.Workbench.ProgressMonitors.GetSaveProgressMonitor (false)) {
-						ce.Save (mon);
-					}
+					IdeApp.ProjectOperations.Save (ce);
+					return;
 				}
 			}
 		}
