@@ -36,10 +36,11 @@ using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Search;
 using MonoDevelop.Ide.Commands;
 using MonoDevelop.Components.Commands;
+using MonoDevelop.Core.Gui;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Projects;
 using MonoDevelop.Projects.Text;
-using MonoDevelop.Projects.Parser;
+using MonoDevelop.Projects.Dom;
 using MonoDevelop.DesignerSupport.Toolbox; 
 using MonoDevelop.DesignerSupport.PropertyGrid;
 using MonoDevelop.Deployment;
@@ -49,7 +50,7 @@ using Gdk;
 
 namespace MonoDevelop.GtkCore.GuiBuilder
 {
-	public class GuiBuilderView : CombinedDesignView, IToolboxConsumer
+	public class GuiBuilderView : CombinedDesignView, IToolboxConsumer, MonoDevelop.DesignerSupport.IOutlinedDocument
 	{
 		Stetic.WidgetDesigner designer;
 		Stetic.ActionGroupDesigner actionsBox;
@@ -68,7 +69,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		{
 			rootName = window.Name;
 			gproject = window.Project;
-			GtkDesignInfo info = GtkCoreService.GetGtkInfo (gproject.Project);
+			GtkDesignInfo info = GtkDesignInfo.FromProject (gproject.Project);
 			gproject.SteticProject.ImagesRootPath = FileService.AbsoluteToRelativePath (info.GtkGuiFolder, gproject.Project.BaseDirectory);
 			gproject.UpdateLibraries ();
 			LoadDesigner ();
@@ -84,7 +85,6 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			
 			gproject.Unloaded += OnDisposeProject;
 			
-			GtkDesignInfo info = GtkCoreService.GetGtkInfo (gproject.Project);
 			designer = gproject.SteticProject.CreateWidgetDesigner (window.RootWidget, false);
 			if (designer.RootComponent == null) {
 				// Something went wrong while creating the designer. Show it, but don't do aything else.
@@ -93,7 +93,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				return;
 			}
 
-			designer.AllowWidgetBinding = (info != null && !info.GeneratePartialClasses);
+			designer.AllowWidgetBinding = !gproject.Project.UsePartialTypes;
 			
 			codeBinder = new CodeBinder (gproject.Project, new OpenDocumentFileProvider (), designer.RootComponent);
 			
@@ -116,7 +116,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			
 			// Actions designer
 			actionsBox = designer.CreateActionGroupDesigner ();
-			actionsBox.AllowActionBinding = (info != null && !info.GeneratePartialClasses);
+			actionsBox.AllowActionBinding = !gproject.Project.UsePartialTypes;
 			actionsBox.BindField += new EventHandler (OnBindActionField);
 			actionsBox.ModifiedChanged += new EventHandler (OnActionshanged);
 			
@@ -194,13 +194,12 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		public override void ShowPage (int npage)
 		{
-			if (designer != null && window != null && !ErrorMode) {
+			if (npage == 0 && designer != null && window != null && !ErrorMode) {
 				// At every page switch update the generated code, to make sure code completion works
 				// for the generated fields. The call to GenerateSteticCodeStructure will generate
 				// the code for the window (only the fields in fact) and update the parser database, it
 				// will not save the code to disk.
-				GtkDesignInfo info = GtkCoreService.GetGtkInfo (gproject.Project);
-				if (info != null && info.GeneratePartialClasses)
+				if (gproject.Project.UsePartialTypes)
 					GuiBuilderService.GenerateSteticCodeStructure ((DotNetProject)gproject.Project, designer.RootComponent, null, false, false);
 			}
 			base.ShowPage (npage);
@@ -224,14 +223,13 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				// way the component that has been renamed will be generated with the
 				// old name, and UpdateField will be able to find it (to rename the
 				// references to the field, it needs to have the old name).
-				GtkDesignInfo info = GtkCoreService.GetGtkInfo (gproject.Project);
-				if (info != null && info.GeneratePartialClasses)
+				if (gproject.Project.UsePartialTypes)
 					GuiBuilderService.GenerateSteticCodeStructure ((DotNetProject)gproject.Project, designer.RootComponent, args, false, false);
 				
 				codeBinder.UpdateField (args.Component, args.OldName);
 			}
 			catch (Exception ex) {
-				IdeApp.Services.MessageService.ShowError (ex);
+				MessageService.ShowException (ex);
 			}
 		}
 		
@@ -289,7 +287,6 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			if (designer == null)
 				return;
 			
-			string oldName = window.RootWidget.Name;
 			string oldBuildFile = GuiBuilderService.GetBuildCodeFileName (gproject.Project, window.RootWidget.Name);
 			
 			codeBinder.UpdateBindings (fileName);
@@ -309,21 +306,6 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			}
 			
 			gproject.Save (true);
-			
-			if (window.RootWidget.Name != oldName) {
-				// The name of the component has changed. If this component is being
-				// exported by the library, then the component reference also has to
-				// be updated in the project configuration
-				
-				GtkDesignInfo info = GtkCoreService.GetGtkInfo (gproject.Project);
-				if (info.IsExported (oldName)) {
-					info.RemoveExportedWidget (oldName);
-					info.AddExportedWidget (codeBinder.TargetObject.Name);
-					info.UpdateGtkFolder ();
-					GtkCoreService.UpdateObjectsFile (gproject.Project);
-					IdeApp.ProjectOperations.SaveProject (gproject.Project);
-				}
-			}
 		}
 		
 		public override bool IsDirty {
@@ -339,13 +321,13 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		public override void JumpToSignalHandler (Stetic.Signal signal)
 		{
-			IClass cls = codeBinder.GetClass ();
+			IType cls = codeBinder.GetClass ();
 			if (cls == null)
 				return;
 			foreach (IMethod met in cls.Methods) {
 				if (met.Name == signal.Handler) {
 					ShowPage (0);
-					JumpTo (met.Region.BeginLine, met.Region.BeginColumn);
+					JumpTo (met.Location.Line, met.Location.Column);
 					break;
 				}
 			}
@@ -436,7 +418,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 					asm = Runtime.SystemAssemblyService.GetAssemblyFullName (asm);
 					if (asm == null)
 						return;
-					if (Runtime.SystemAssemblyService.GetPackageFromFullName (asm) != null) {
+					if (Runtime.SystemAssemblyService.GetPackagesFromFullName (asm).Length > 0) {
 						pref = new ProjectReference (ReferenceType.Gac, asm);
 					} else {
 						asm = Runtime.SystemAssemblyService.GetAssemblyLocation (asm);
@@ -446,11 +428,21 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				else
 					pref = new ProjectReference (node.ReferenceType, node.Reference);
 				
-				foreach (ProjectReference pr in gproject.Project.ProjectReferences) {
+				foreach (ProjectReference pr in gproject.Project.References) {
 					if (pr.Reference == pref.Reference)
 						return;
 				}
-				gproject.Project.ProjectReferences.Add (pref);
+				gproject.Project.References.Add (pref);
+		}
+		
+		Widget MonoDevelop.DesignerSupport.IOutlinedDocument.GetOutlineWidget ()
+		{
+			return GuiBuilderDocumentOutline.Instance;
+		}
+
+		void MonoDevelop.DesignerSupport.IOutlinedDocument.ReleaseOutlineWidget ()
+		{
+			//Do nothing. We keep the instance to avoid creation cost when switching documents.
 		}
 
 		Gtk.TargetEntry[] targets = new Gtk.TargetEntry[] {
