@@ -33,6 +33,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 		}
 		
 		public abstract void config_ensure (string config_dir, IntPtr pool);
+		public abstract void config_get_config (ref IntPtr cfg_hash, string config_dir, IntPtr pool);
 		public abstract void auth_open (out IntPtr auth_baton, IntPtr providers, IntPtr pool);
 		public abstract void auth_set_parameter (IntPtr auth_baton, string name, IntPtr value);
 		public abstract IntPtr auth_get_parameter (IntPtr auth_baton, string name);
@@ -98,6 +99,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 		public abstract IntPtr client_revert (IntPtr apr_array_header_t_targets, int recursive,
 		                                      IntPtr ctx, IntPtr pool);
 		
+		public abstract IntPtr client_resolved (string path, int recursive, IntPtr ctx, IntPtr pool);
+		
 		public abstract IntPtr client_move (ref IntPtr commit_info_p, string srcPath, ref Rev rev,
 		                                    string destPath, int force, IntPtr ctx, IntPtr pool);
 		
@@ -113,6 +116,24 @@ namespace MonoDevelop.VersionControl.Subversion {
 		                                    IntPtr ctx, IntPtr pool);
 		
 		
+		public abstract IntPtr client_merge_peg2 (string source,
+		                                          ref Rev revision1,
+		                                          ref Rev revision2,
+		                                          ref Rev peg_revision,
+		                                          string target_wcpath,
+		                                          bool recurse,
+		                                          bool ignore_ancestry,
+		                                          bool force,
+		                                          bool dry_run,
+		                                          IntPtr merge_options,
+		                                          IntPtr ctx,
+		                                          IntPtr pool);
+		
+		public abstract IntPtr client_lock (IntPtr apr_array_header_t_targets, string comment, int steal_lock, IntPtr ctx, IntPtr pool);
+		
+		public abstract IntPtr client_unlock (IntPtr apr_array_header_t_targets, int break_lock, IntPtr ctx, IntPtr pool);
+		
+		public abstract IntPtr client_prop_get (out IntPtr value, string name, string target, ref Rev revision, int recurse, IntPtr ctx, IntPtr pool);
 		
 		public class DirEnt {
 			public readonly string Name;
@@ -163,12 +184,17 @@ namespace MonoDevelop.VersionControl.Subversion {
   			public readonly DateTime LastCommitDate;
   			public readonly string LastCommitAuthor;
  			
-			public readonly VersionStatus TextStatus;
-			public readonly VersionStatus PropsStatus;
+			public readonly LibSvnClient.VersionStatus TextStatus;
+			public readonly LibSvnClient.VersionStatus PropsStatus;
 			public readonly bool Locked;
 			public readonly bool Switched;
-			public readonly VersionStatus RemoteTextStatus;
-			public readonly VersionStatus RemotePropsStatus;
+			public readonly LibSvnClient.VersionStatus RemoteTextStatus;
+			public readonly LibSvnClient.VersionStatus RemotePropsStatus;
+			public readonly bool RepoLocked;
+			public readonly bool LockOwned;
+			public readonly string LockToken;
+			public readonly string LockOwner;
+			public readonly string LockComment;
 			
 			static readonly DateTime Epoch = new DateTime (1970, 1, 1);
 			
@@ -182,7 +208,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 				Switched = status.switched != 0;
 				RemoteTextStatus = (VersionStatus) status.svn_wc_status_kind_text_repo;
 				RemotePropsStatus = (VersionStatus) status.svn_wc_status_kind_props_repo;
-				
+
 				if (status.to_svn_wc_entry_t == IntPtr.Zero)
 					return;
 				
@@ -210,6 +236,21 @@ namespace MonoDevelop.VersionControl.Subversion {
 	  			LastCommitRevision = ent.last_commit_rev;
 	  			LastCommitDate = Epoch.AddTicks (ent.last_commit_date * 10);
 	  			LastCommitAuthor = ent.last_commit_author;
+				LockToken = ent.lock_token;
+				LockOwner = ent.lock_owner;
+				LockComment = ent.lock_comment;
+				
+				if (status.repos_lock != IntPtr.Zero) {
+					svn_lock_t repoLock = (svn_lock_t) Marshal.PtrToStructure (status.repos_lock, typeof (svn_lock_t));
+					LockToken = repoLock.token;
+					LockOwner = repoLock.owner;
+					LockComment = repoLock.comment;
+					RepoLocked = true;
+				}
+				if (LockToken != null) {
+					LockOwned = true;
+					RepoLocked = true;
+				}
 			}
 		}
 		
@@ -259,6 +300,27 @@ namespace MonoDevelop.VersionControl.Subversion {
 			public IntPtr notify_baton;
 			public svn_client_get_commit_log_t LogMsgFunc;
 			public IntPtr logmsg_baton;
+			public IntPtr config;
+			public IntPtr cancel_func;
+			public IntPtr cancel_baton;
+			public svn_wc_notify_func2_t NotifyFunc2;
+			public IntPtr notify_baton2;
+		}
+		
+		public struct svn_wc_notify_t {
+			public IntPtr path;
+			public LibSvnClient.NotifyAction action;
+			public LibSvnClient.NodeKind kind;
+			public IntPtr mime_type;
+			public IntPtr repo_lock;
+			public IntPtr err;
+			public LibSvnClient.NotifyState content_state;
+			public LibSvnClient.NotifyState prop_state;
+			public LibSvnClient.NotifyLockState lock_state;
+			public long revision;
+			public IntPtr changelist_name;
+			public IntPtr merge_range;
+			public IntPtr path_prefix;
 		}
 		
 		public struct svn_error_t {
@@ -354,17 +416,34 @@ namespace MonoDevelop.VersionControl.Subversion {
 			public int last_commit_rev;
 			public long last_commit_date;
 			public string last_commit_author;
+			public string lock_token;
+			public string lock_owner;
+			public string lock_comment;
 		}
 		
 		public struct svn_wc_status_t {
 			public IntPtr to_svn_wc_entry_t;
-			public int svn_wc_status_kind_text;
-			public int svn_wc_status_kind_props;
+			public int svn_wc_status_kind_text;      // Field not shown in the documentation
+			public int svn_wc_status_kind_props;     // Field not shown in the documentation
 			public int locked;
 			public int copied;
 			public int switched;
-			public int svn_wc_status_kind_text_repo;
-			public int svn_wc_status_kind_props_repo;
+			public int svn_wc_status_kind_text_repo;  // Field not shown in the documentation
+			public int svn_wc_status_kind_props_repo; // Field not shown in the documentation
+			public IntPtr repos_lock;
+		}
+		
+		public struct svn_lock_t {
+			public string path;
+			public string token;
+			public string owner;
+			public string comment;
+			public int is_dav_comment;
+		}
+		
+		public struct svn_string_t {
+			public IntPtr data;
+			public int len;
 		}
 		
 		public struct Rev {
@@ -460,7 +539,11 @@ namespace MonoDevelop.VersionControl.Subversion {
 			CommitDeleted,
 			CommitReplaced,
 			CommitPostfixTxDelta,
-			BlameRevision
+			BlameRevision,
+			Locked,
+			Unlocked,
+			FailedLock,
+			FailedUnlock
 		}
 		
 		public enum NotifyState {
@@ -474,6 +557,12 @@ namespace MonoDevelop.VersionControl.Subversion {
 			Conflicted
 		}
 		
+		public enum NotifyLockState {
+			Unchanged = 2,
+			Locked = 3,
+			Unlocked = 4
+		}
+		
 		public delegate void svn_wc_status_func_t (IntPtr baton, IntPtr path, ref svn_wc_status_t status);
 		
 		// Certificate is not yet valid.
@@ -503,6 +592,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 		                                           IntPtr mime_type, NotifyState content_state, NotifyState prop_state,
 		                                           long revision);
 		
+		public delegate void svn_wc_notify_func2_t (IntPtr baton, ref svn_wc_notify_t notify, IntPtr pool);
+		
 		public delegate IntPtr svn_client_get_commit_log_t (ref IntPtr log_msg, ref IntPtr tmp_file, IntPtr commit_items,
 		                                                    IntPtr baton, IntPtr pool);
 	}
@@ -515,6 +606,11 @@ namespace MonoDevelop.VersionControl.Subversion {
 			svn_config_ensure (config_dir, pool);
 		}
 		
+		public override void config_get_config (ref IntPtr cfg_hash, string config_dir, IntPtr pool)
+		{
+			svn_config_get_config (ref cfg_hash, config_dir, pool);
+		}
+		
 		public override void auth_open (out IntPtr auth_baton, IntPtr providers, IntPtr pool)	
 		{
 			svn_auth_open (out auth_baton, providers, pool);
@@ -680,6 +776,11 @@ namespace MonoDevelop.VersionControl.Subversion {
 			return svn_client_revert (apr_array_header_t_targets, recursive, ctx, pool);
 		}
 		
+		public override IntPtr client_resolved (string path, int recursive, IntPtr ctx, IntPtr pool)
+		{
+			return svn_client_resolved (path, recursive, ctx, pool);
+		}
+		
 		public override IntPtr client_move (ref IntPtr commit_info_p, string srcPath, ref Rev rev,
 		                                    string destPath, int force, IntPtr ctx, IntPtr pool)
 		{
@@ -707,7 +808,43 @@ namespace MonoDevelop.VersionControl.Subversion {
 			                        no_diff_deleted, outfile, errfile, ctx, pool);
 		}
 		
+		public override IntPtr client_merge_peg2 (
+		                          string source,
+		                          ref Rev revision1,
+		                          ref Rev revision2,
+		                          ref Rev peg_revision,
+		                          string target_wcpath,
+		                          bool recurse,
+		                          bool ignore_ancestry,
+		                          bool force,
+		                          bool dry_run,
+		                          IntPtr merge_options,
+		                          IntPtr ctx,
+		                          IntPtr pool)
+		{
+			// svn_boolean_t == int
+			return svn_client_merge_peg2 (source, ref revision1, ref revision2, ref peg_revision, target_wcpath, 
+			                              recurse ? 1: 0, ignore_ancestry ? 1 : 0, force ? 1 : 0, dry_run ? 1 : 0,
+			                              merge_options, ctx, pool);
+		}
+		
+		public override IntPtr client_lock (IntPtr apr_array_header_t_targets, string comment, int steal_lock, IntPtr ctx, IntPtr pool)
+		{
+			return svn_client_lock (apr_array_header_t_targets, comment, steal_lock, ctx, pool);
+		}
+		
+		public override IntPtr client_unlock (IntPtr apr_array_header_t_targets, int break_lock, IntPtr ctx, IntPtr pool)
+		{
+			return svn_client_unlock (apr_array_header_t_targets, break_lock, ctx, pool);
+		}
+		
+		public override IntPtr client_prop_get (out IntPtr value, string name, string target, ref Rev revision, int recurse, IntPtr ctx, IntPtr pool)
+		{
+			return svn_client_prop_get (out value, name, target, ref revision, recurse, ctx, pool);
+		}
+		
 		[DllImport(svnclientlib)] static extern void svn_config_ensure (string config_dir, IntPtr pool);
+		[DllImport(svnclientlib)] static extern void svn_config_get_config (ref IntPtr cfg_hash, string config_dir, IntPtr pool);
 		[DllImport(svnclientlib)] static extern void svn_auth_open (out IntPtr auth_baton, IntPtr providers, IntPtr pool);
 		[DllImport(svnclientlib)] static extern void svn_auth_set_parameter (IntPtr auth_baton, string name, IntPtr value);
 		[DllImport(svnclientlib)] static extern IntPtr svn_auth_get_parameter (IntPtr auth_baton, string name);
@@ -773,6 +910,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 		[DllImport(svnclientlib)] static extern IntPtr svn_client_revert (IntPtr apr_array_header_t_targets, int recursive,
 		                                                                  IntPtr ctx, IntPtr pool);
 		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_resolved (string path, int recursive, IntPtr ctx, IntPtr pool);
+		
 		[DllImport(svnclientlib)] static extern IntPtr svn_client_move (ref IntPtr commit_info_p, string srcPath, ref Rev rev,
 		                                                                string destPath, int force, IntPtr ctx, IntPtr pool);
 		
@@ -790,6 +929,25 @@ namespace MonoDevelop.VersionControl.Subversion {
 		                                                                IntPtr errfile,
 		                                                                IntPtr ctx,
 		                                                                IntPtr pool);
+		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_merge_peg2 (string source,
+		                                                                      ref Rev revision1,
+		                                                                      ref Rev revision2,
+		                                                                      ref Rev peg_revision,
+		                                                                      string target_wcpath,
+		                                                                      int recurse,
+		                                                                      int ignore_ancestry,
+		                                                                      int force,
+		                                                                      int dry_run,
+		                                                                      IntPtr merge_options,
+		                                                                      IntPtr ctx,
+		                                                                      IntPtr pool);
+		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_lock (IntPtr apr_array_header_t_targets, string comment, int steal_lock, IntPtr ctx, IntPtr pool);
+		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_unlock (IntPtr apr_array_header_t_targets, int break_lock, IntPtr ctx, IntPtr pool);
+		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_prop_get (out IntPtr value, string name, string target, ref Rev revision, int recurse, IntPtr ctx, IntPtr pool);
 	}
 	
 	public class LibSvnClient1 : LibSvnClient {
@@ -800,6 +958,11 @@ namespace MonoDevelop.VersionControl.Subversion {
 			svn_config_ensure (config_dir, pool);
 		}
 		
+		public override void config_get_config (ref IntPtr cfg_hash, string config_dir, IntPtr pool)
+		{
+			svn_config_get_config (ref cfg_hash, config_dir, pool);
+		}
+		
 		public override void auth_open (out IntPtr auth_baton, IntPtr providers, IntPtr pool)	
 		{
 			svn_auth_open (out auth_baton, providers, pool);
@@ -965,6 +1128,11 @@ namespace MonoDevelop.VersionControl.Subversion {
 			return svn_client_revert (apr_array_header_t_targets, recursive, ctx, pool);
 		}
 		
+		public override IntPtr client_resolved (string path, int recursive, IntPtr ctx, IntPtr pool)
+		{
+			return svn_client_resolved (path, recursive, ctx, pool);
+		}
+		
 		public override IntPtr client_move (ref IntPtr commit_info_p, string srcPath, ref Rev rev,
 		                                    string destPath, int force, IntPtr ctx, IntPtr pool)
 		{
@@ -992,7 +1160,43 @@ namespace MonoDevelop.VersionControl.Subversion {
 			                        no_diff_deleted, outfile, errfile, ctx, pool);
 		}
 		
+		public override IntPtr client_merge_peg2 (
+		                          string source,
+		                          ref Rev revision1,
+		                          ref Rev revision2,
+		                          ref Rev peg_revision,
+		                          string target_wcpath,
+		                          bool recurse,
+		                          bool ignore_ancestry,
+		                          bool force,
+		                          bool dry_run,
+		                          IntPtr merge_options,
+		                          IntPtr ctx,
+		                          IntPtr pool)
+		{
+			// svn_boolean_t == int
+			return svn_client_merge_peg2 (source, ref revision1, ref revision2, ref peg_revision, target_wcpath, 
+			                              recurse ? 1: 0, ignore_ancestry ? 1 : 0, force ? 1 : 0, dry_run ? 1 : 0,
+			                              merge_options, ctx, pool);
+		}
+		
+		public override IntPtr client_lock (IntPtr apr_array_header_t_targets, string comment, int steal_lock, IntPtr ctx, IntPtr pool)
+		{
+			return svn_client_lock (apr_array_header_t_targets, comment, steal_lock, ctx, pool);
+		}
+		
+		public override IntPtr client_unlock (IntPtr apr_array_header_t_targets, int break_lock, IntPtr ctx, IntPtr pool)
+		{
+			return svn_client_unlock (apr_array_header_t_targets, break_lock, ctx, pool);
+		}
+		
+		public override IntPtr client_prop_get (out IntPtr value, string name, string target, ref Rev revision, int recurse, IntPtr ctx, IntPtr pool)
+		{
+			return svn_client_prop_get (out value, name, target, ref revision, recurse, ctx, pool);
+		}
+
 		[DllImport(svnclientlib)] static extern void svn_config_ensure (string config_dir, IntPtr pool);
+		[DllImport(svnclientlib)] static extern void svn_config_get_config (ref IntPtr cfg_hash, string config_dir, IntPtr pool);
 		[DllImport(svnclientlib)] static extern void svn_auth_open (out IntPtr auth_baton, IntPtr providers, IntPtr pool);
 		[DllImport(svnclientlib)] static extern void svn_auth_set_parameter (IntPtr auth_baton, string name, IntPtr value);
 		[DllImport(svnclientlib)] static extern IntPtr svn_auth_get_parameter (IntPtr auth_baton, string name);
@@ -1058,6 +1262,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 		[DllImport(svnclientlib)] static extern IntPtr svn_client_revert (IntPtr apr_array_header_t_targets, int recursive,
 		                                                                  IntPtr ctx, IntPtr pool);
 		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_resolved (string path, int recursive, IntPtr ctx, IntPtr pool);
+		
 		[DllImport(svnclientlib)] static extern IntPtr svn_client_move (ref IntPtr commit_info_p, string srcPath, ref Rev rev,
 		                                                                string destPath, int force, IntPtr ctx, IntPtr pool);
 		
@@ -1075,5 +1281,25 @@ namespace MonoDevelop.VersionControl.Subversion {
 		                                                                IntPtr errfile,
 		                                                                IntPtr ctx,
 		                                                                IntPtr pool);
+		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_merge_peg2 (string source,
+		                                                                      ref Rev revision1,
+		                                                                      ref Rev revision2,
+		                                                                      ref Rev peg_revision,
+		                                                                      string target_wcpath,
+		                                                                      int recurse,
+		                                                                      int ignore_ancestry,
+		                                                                      int force,
+		                                                                      int dry_run,
+		                                                                      IntPtr merge_options,
+		                                                                      IntPtr ctx,
+		                                                                      IntPtr pool);
+		
+		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_lock (IntPtr apr_array_header_t_targets, string comment, int steal_lock, IntPtr ctx, IntPtr pool);
+		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_unlock (IntPtr apr_array_header_t_targets, int break_lock, IntPtr ctx, IntPtr pool);
+		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_prop_get (out IntPtr value, string name, string target, ref Rev revision, int recurse, IntPtr ctx, IntPtr pool);
 	}
 }
