@@ -25,6 +25,9 @@ namespace MonoDevelop.VersionControl.Subversion {
 		ArrayList updateFileList;
 		string commitmessage = null;
 		
+		ArrayList lockFileList;
+		LibSvnClient.NotifyLockState requiredLockState;
+		
 		// retain this so the delegates aren't GC'ed
 		LibSvnClient.svn_client_ctx_t ctxstruct;
 		
@@ -115,9 +118,12 @@ namespace MonoDevelop.VersionControl.Subversion {
 			// calls because it needs to be allocated by SVN.  I hope
 			// this doesn't cause any memory leaks.
 			ctxstruct = new LibSvnClient.svn_client_ctx_t ();
-			ctxstruct.NotifyFunc = new LibSvnClient.svn_wc_notify_func_t (svn_wc_notify_func_t_impl);
+			ctxstruct.NotifyFunc2 = new LibSvnClient.svn_wc_notify_func2_t (svn_wc_notify_func_t_impl);
 			ctxstruct.LogMsgFunc = new LibSvnClient.svn_client_get_commit_log_t (svn_client_get_commit_log_impl);
 			
+			// Load user and system configuration
+			svn.config_get_config (ref ctxstruct.config, null, pool);
+			
 			IntPtr providers = apr.array_make (pool, 1, IntPtr.Size);
 			IntPtr item;
 			
@@ -473,6 +479,29 @@ namespace MonoDevelop.VersionControl.Subversion {
 			}
 		}
 		
+		public void Resolve (string path, bool recurse, IProgressMonitor monitor)
+		{
+			if (path == null || monitor == null)
+				throw new ArgumentNullException();
+			
+			lock (sync) {
+				if (inProgress)
+					throw new SubversionException("Another Subversion operation is already in progress.");
+				inProgress = true;
+			}
+			
+			updatemonitor = monitor;
+			IntPtr localpool = newpool (pool);
+			
+			try {
+				CheckError (svn.client_resolved (path, recurse ? 1 : 0, ctx, localpool));
+			} finally {
+				apr.pool_destroy (localpool);
+				updatemonitor = null;
+				inProgress = false;
+			}
+		}
+		
 		public void Add (string path, bool recurse, IProgressMonitor monitor)
 		{
 			if (path == null || monitor == null)
@@ -547,7 +576,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 				CheckError (svn.client_commit (ref commit_info, array, 0, ctx, localpool));
 				unsafe {
 					if (commit_info != IntPtr.Zero)
-						monitor.Log.WriteLine ("Revision: " + ((LibSvnClient.svn_client_commit_info_t *) commit_info.ToPointer())->revision);
+						monitor.Log.WriteLine (GettextCatalog.GetString ("Revision: {0}", ((LibSvnClient.svn_client_commit_info_t *) commit_info.ToPointer())->revision));
 				}
 			} finally {
 				commitmessage = null;
@@ -646,6 +675,68 @@ namespace MonoDevelop.VersionControl.Subversion {
 			}
 		}
 		
+		public void Lock (IProgressMonitor monitor, string comment, bool stealLock, params string[] paths)
+		{
+			lock (sync) {
+				if (inProgress)
+					throw new SubversionException ("Another Subversion operation is already in progress.");
+				inProgress = true;
+			}
+			
+			IntPtr localpool = newpool (pool);
+			IntPtr array = apr.array_make (localpool, 0, IntPtr.Size);
+			updatemonitor = monitor;
+
+			try {
+				foreach (string path in paths) {
+					IntPtr item = apr.array_push (array);
+					Marshal.WriteIntPtr (item, apr.pstrdup (localpool, path));
+				}
+				lockFileList = new ArrayList ();
+				requiredLockState = LibSvnClient.NotifyLockState.Locked;
+				
+				CheckError (svn.client_lock (array, comment, stealLock ? 1 : 0, ctx, localpool));
+				if (paths.Length != lockFileList.Count)
+					throw new SubversionException ("Lock operation failed.");
+			} finally {
+				apr.pool_destroy (localpool);
+				lockFileList = null;
+				updatemonitor = null;
+				inProgress = false;
+			}
+		}
+		
+		public void Unlock (IProgressMonitor monitor, bool breakLock, params string[] paths)
+		{
+			lock (sync) {
+				if (inProgress)
+					throw new SubversionException ("Another Subversion operation is already in progress.");
+				inProgress = true;
+			}
+			
+			IntPtr localpool = newpool (pool);
+			IntPtr array = apr.array_make (localpool, 0, IntPtr.Size);
+			updatemonitor = monitor;
+			
+			try {
+				foreach (string path in paths) {
+					IntPtr item = apr.array_push (array);
+					Marshal.WriteIntPtr (item, apr.pstrdup (localpool, path));
+				}
+				lockFileList = new ArrayList ();
+				requiredLockState = LibSvnClient.NotifyLockState.Unlocked;
+			
+				CheckError (svn.client_unlock (array, breakLock ? 1 : 0, ctx, localpool));
+				if (paths.Length != lockFileList.Count)
+					throw new SubversionException ("Lock operation failed.");
+			} finally {
+				apr.pool_destroy (localpool);
+				lockFileList = null;
+				updatemonitor = null;
+				inProgress = false;
+			}
+		}
+		
 		public string PathDiff (string path1, LibSvnClient.Rev revision1, string path2, LibSvnClient.Rev revision2, bool recursive)
 		{
 			IntPtr localpool = newpool (pool);
@@ -691,6 +782,31 @@ namespace MonoDevelop.VersionControl.Subversion {
 				} catch {}
 			}
 		}
+
+		public void RevertToRevision (string path, int revision)
+		{
+			Merge (path, LibSvnClient.Rev.Head, LibSvnClient.Rev.Number(revision));
+		}
+		
+		public void RevertRevision (string path, int revision)
+		{
+			Merge (path, LibSvnClient.Rev.Number(revision), LibSvnClient.Rev.Number (revision - 1));
+		}
+		
+		private void Merge (string path, LibSvnClient.Rev revision1, LibSvnClient.Rev revision2)
+		{
+			IntPtr localpool = newpool (pool);
+			LibSvnClient.Rev working = LibSvnClient.Rev.Working;
+			CheckError (svn.client_merge_peg2 (path, 
+			                                   ref revision1,
+			                                   ref revision2,
+			                                   ref working,
+			                                   path,
+			                                   false, false, false, false,
+			                                   IntPtr.Zero, //default options is NULL
+			                                   ctx, localpool));
+			return;
+		}
 		
 		IntPtr svn_client_get_commit_log_impl (ref IntPtr log_msg, ref IntPtr tmp_file,
 		                                       IntPtr commit_items, IntPtr baton, IntPtr pool)
@@ -721,28 +837,30 @@ namespace MonoDevelop.VersionControl.Subversion {
 			throw new SubversionException (msg);
 		}
 		
-		void svn_wc_notify_func_t_impl (IntPtr baton, IntPtr path, LibSvnClient.NotifyAction action,
-		                                LibSvnClient.NodeKind kind, IntPtr mime_type,
-		                                LibSvnClient.NotifyState content_state,
-		                                LibSvnClient.NotifyState prop_state,
-		                                long revision)
+		void svn_wc_notify_func_t_impl (IntPtr baton, ref LibSvnClient.svn_wc_notify_t data, IntPtr pool)
 		{
-			string actiondesc = action.ToString ();
-			string file = Marshal.PtrToStringAnsi (path);
+			string actiondesc = data.action.ToString ();
+			string file = Marshal.PtrToStringAnsi (data.path);
 			bool notifyChange = false;
 			
-			switch (action) {
-			case LibSvnClient.NotifyAction.UpdateAdd: actiondesc = "Added"; break;
-			case LibSvnClient.NotifyAction.UpdateDelete: actiondesc = "Deleted"; break;
-			case LibSvnClient.NotifyAction.UpdateUpdate: actiondesc = "Updating"; notifyChange = true; break;
-			case LibSvnClient.NotifyAction.UpdateExternal: actiondesc = "External Updated"; break;
-			case LibSvnClient.NotifyAction.UpdateCompleted: actiondesc = "Finished"; break;
+			switch (data.action) {
+			case LibSvnClient.NotifyAction.UpdateAdd: actiondesc = GettextCatalog.GetString ("Added"); break;
+			case LibSvnClient.NotifyAction.UpdateDelete: actiondesc = GettextCatalog.GetString ("Deleted"); break;
+			case LibSvnClient.NotifyAction.UpdateUpdate: actiondesc = GettextCatalog.GetString ("Updating"); notifyChange = true; break;
+			case LibSvnClient.NotifyAction.UpdateExternal: actiondesc = GettextCatalog.GetString ("External Updated"); break;
+			case LibSvnClient.NotifyAction.UpdateCompleted: actiondesc = GettextCatalog.GetString ("Finished"); break;
 				
-			case LibSvnClient.NotifyAction.CommitAdded: actiondesc = "Added"; break;
-			case LibSvnClient.NotifyAction.CommitDeleted: actiondesc = "Deleted"; break;
-			case LibSvnClient.NotifyAction.CommitModified: actiondesc = "Modified"; notifyChange = true; break;
-			case LibSvnClient.NotifyAction.CommitReplaced: actiondesc = "Replaced"; notifyChange = true; break;
-			case LibSvnClient.NotifyAction.CommitPostfixTxDelta: actiondesc = "Sending Content"; break;
+			case LibSvnClient.NotifyAction.CommitAdded: actiondesc = GettextCatalog.GetString ("Added"); break;
+			case LibSvnClient.NotifyAction.CommitDeleted: actiondesc = GettextCatalog.GetString ("Deleted"); break;
+			case LibSvnClient.NotifyAction.CommitModified: actiondesc = GettextCatalog.GetString ("Modified"); notifyChange = true; break;
+			case LibSvnClient.NotifyAction.CommitReplaced: actiondesc = GettextCatalog.GetString ("Replaced"); notifyChange = true; break;
+			case LibSvnClient.NotifyAction.CommitPostfixTxDelta: actiondesc = GettextCatalog.GetString ("Sending Content"); break;
+					
+			case LibSvnClient.NotifyAction.Locked: actiondesc = GettextCatalog.GetString ("Locked"); break;
+			case LibSvnClient.NotifyAction.Unlocked: actiondesc = GettextCatalog.GetString ("Unlocked"); break;
+			case LibSvnClient.NotifyAction.FailedLock: actiondesc = GettextCatalog.GetString ("Lock Failed"); break;
+			case LibSvnClient.NotifyAction.FailedUnlock: actiondesc = GettextCatalog.GetString ("Unlock Failed"); break;
+					
 				/*Add,
 				 Copy,
 				 Delete,
@@ -760,6 +878,9 @@ namespace MonoDevelop.VersionControl.Subversion {
 				updatemonitor.Log.WriteLine (actiondesc + " " + file);
 			if (updateFileList != null && notifyChange && File.Exists (file))
 				updateFileList.Add (file);
+			
+			if (lockFileList != null && data.lock_state == requiredLockState)
+				lockFileList.Add (file);
 		}
 		
 		public class StatusCollector {
