@@ -4,6 +4,7 @@ using System.Collections.Specialized;
 using System.Collections;
 using Gtk;
 using MonoDevelop.Core;
+using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Gui;
 using Mono.Addins;
 
@@ -50,19 +51,23 @@ namespace MonoDevelop.VersionControl.Dialogs
 			foreach (object ob in exts) {
 				CommitDialogExtension ext = ob as CommitDialogExtension;
 				if (ext == null) {
-					IdeApp.Services.MessageService.ShowError ("Commit extension type " + ob.GetType() + " must be a subclass of CommitDialogExtension");
+					MessageService.ShowError ("Commit extension type " + ob.GetType() + " must be a subclass of CommitDialogExtension");
 					continue;
 				}
 				vboxExtensions.PackEnd (ext, false, false, 0);
 				ext.Initialize (changeSet);
 				extensions.Add (ext);
+				ext.AllowCommitChanged += HandleAllowCommitChanged;
 			}
+			HandleAllowCommitChanged (null, null);
 
 			foreach (ChangeSetItem info in changeSet.Items) {
 				Gdk.Pixbuf statusicon = VersionControlService.LoadIconForStatus (info.Status);
 				string lstatus = VersionControlService.GetStatusLabel (info.Status);
 				
-				string localpath = info.LocalPath.Substring (changeSet.BaseLocalPath.Length);
+				string localpath = (info.LocalPath.Length <= changeSet.BaseLocalPath.Length)?
+				                    ".":
+				                    info.LocalPath.Substring (changeSet.BaseLocalPath.Length); 
 				if (localpath.Length > 0 && localpath[0] == System.IO.Path.DirectorySeparatorChar) localpath = localpath.Substring(1);
 				if (localpath == "") { localpath = "."; } // not sure if this happens
 				
@@ -70,13 +75,24 @@ namespace MonoDevelop.VersionControl.Dialogs
 				selected.Add (info.LocalPath);
 			}
 			
-			if (changeSet.GlobalComment.Length == 0)
-				Message = changeSet.GenerateGlobalComment (70);
+			if (changeSet.GlobalComment.Length == 0) {
+				AuthorInformation aInfo;
+				CommitMessageFormat fmt = VersionControlService.GetCommitMessageFormat (changeSet, out aInfo);
+				Message = changeSet.GenerateGlobalComment (fmt, aInfo);
+			}
 			else
 				Message = changeSet.GlobalComment;
 				
 			textview.Buffer.Changed += OnTextChanged;
 		}
+
+		void HandleAllowCommitChanged (object sender, EventArgs e)
+		{
+			bool allowCommit = true;
+			foreach (CommitDialogExtension ext in extensions)
+				allowCommit &= ext.AllowCommit;
+			SetResponseSensitive (Gtk.ResponseType.Ok, allowCommit);
+		}
 		
 		protected override void OnResponse (Gtk.ResponseType type)
 		{
@@ -101,7 +117,7 @@ namespace MonoDevelop.VersionControl.Dialogs
 					try {
 						res = ext.OnBeginCommit (changeSet);
 					} catch (Exception ex) {
-						IdeApp.Services.MessageService.ShowError (ex);
+						MessageService.ShowException (ex);
 						res = false;
 					}
 					
@@ -160,11 +176,5 @@ namespace MonoDevelop.VersionControl.Dialogs
 			else
 				selected.Remove (vinfo.LocalPath);
 		}
-		
-		public override void Dispose ()
-		{
-			base.Dispose ();
-			Destroy ();
-		}
 	}
 }
