@@ -21,12 +21,34 @@ namespace MonoDevelop.VersionControl.Views
 			font = Pango.FontDescription.FromString (IdeApp.Services.PlatformService.DefaultMonospaceFont);
 		}
 		
+		void DisposeLayout ()
+		{
+			if (layout != null) {
+				layout.Dispose ();
+				layout = null;
+			}
+		}
+		
+		bool isDisposed = false;
+		protected override void OnDestroyed ()
+		{
+			isDisposed = true;
+			DisposeLayout ();
+			if (font != null) {
+				font.Dispose ();
+				font = null;
+			}
+			base.OnDestroyed ();
+		}
+		
 		public void Reset ()
 		{
 		}
 		
 		public void InitCell (Widget container, bool diffMode, string text, string path)
 		{
+			if (isDisposed)
+				return;
 			this.diffMode = diffMode;
 			
 			if (diffMode) {
@@ -40,6 +62,7 @@ namespace MonoDevelop.VersionControl.Views
 							maxlin = n;
 						}
 					}
+					DisposeLayout ();
 					layout = CreateLayout (container, lines [maxlin]);
 					layout.GetPixelSize (out width, out lineHeight);
 					height = lineHeight * lines.Length;
@@ -48,6 +71,7 @@ namespace MonoDevelop.VersionControl.Views
 					width = height = 0;
 			}
 			else {
+				DisposeLayout ();
 				layout = CreateLayout (container, text);
 				layout.GetPixelSize (out width, out height);
 			}
@@ -68,6 +92,8 @@ namespace MonoDevelop.VersionControl.Views
 
 		protected override void Render (Drawable window, Widget widget, Gdk.Rectangle background_area, Gdk.Rectangle cell_area, Gdk.Rectangle expose_area, CellRendererState flags)
 		{
+			if (isDisposed)
+				return;
 			if (diffMode) {
 				int w, maxy;
 				window.GetSize (out w, out maxy);
@@ -117,6 +143,9 @@ namespace MonoDevelop.VersionControl.Views
 					window.DrawLayout (gc, cell_area.X + 2, y, layout);
 				}
 				window.DrawRectangle (widget.Style.DarkGC (Gtk.StateType.Prelight), false, cell_area.X, recty, cell_area.Width - 1, recth);
+				removedGC.Dispose ();
+				addedGC.Dispose ();
+				infoGC.Dispose ();
 			} else {
 				int y = cell_area.Y + (cell_area.Height - height)/2;
 				window.DrawLayout (widget.Style.TextGC (GetState(flags)), cell_area.X, y, layout);
@@ -143,5 +172,7 @@ namespace MonoDevelop.VersionControl.Views
 			else
 				return StateType.Normal;
 		}
-	}
+	
+		
+}
 }
