@@ -234,19 +234,21 @@ namespace MonoDevelop.Components.Docking
 			return -1;
 		}
 		
-		public void RemoveItemRec (DockItem item)
+		public bool RemoveItemRec (DockItem item)
 		{
 			foreach (DockObject ob in dockObjects) {
 				if (ob is DockGroup) {
-					((DockGroup)ob).RemoveItemRec (item);
+					if (((DockGroup)ob).RemoveItemRec (item))
+						return true;
 				} else {
 					DockGroupItem dit = ob as DockGroupItem;
 					if (dit != null && dit.Item == item) {
 						Remove (ob);
-						return;
+						return true;
 					}
 				}
 			}
+			return false;
 		}
 		
 		public void Remove (DockObject obj)
@@ -637,19 +639,29 @@ namespace MonoDevelop.Components.Docking
 			}
 		}
 		
-		internal void Present (DockItem it)
+		internal void Present (DockItem it, bool giveFocus)
 		{
-			if (type != DockGroupType.Tabbed)
-				return;
-			for (int n=0; n<VisibleObjects.Count; n++) {
-				DockGroupItem dit = VisibleObjects[n] as DockGroupItem;
-				if (dit.Item == it) {
-					currentTabPage = n;
-					if (boundTabStrip != null)
-						boundTabStrip.CurrentPage = it.Widget;
-					return;
+			if (type == DockGroupType.Tabbed) {
+				for (int n=0; n<VisibleObjects.Count; n++) {
+					DockGroupItem dit = VisibleObjects[n] as DockGroupItem;
+					if (dit.Item == it) {
+						currentTabPage = n;
+						if (boundTabStrip != null)
+							boundTabStrip.CurrentPage = it.Widget;
+						break;
+					}
 				}
 			}
+			if (giveFocus && it.Visible)
+				it.SetFocus ();
+		}
+
+		internal bool IsSelectedPage (DockItem it)
+		{
+			if (type != DockGroupType.Tabbed || boundTabStrip == null || boundTabStrip.CurrentTab == -1 || VisibleObjects == null || boundTabStrip.CurrentTab >= VisibleObjects.Count)
+				return false;
+			DockGroupItem dit = VisibleObjects[boundTabStrip.CurrentTab] as DockGroupItem;
+			return dit.Item == it;
 		}
 		
 		internal void UpdateTitle (DockItem it)
