@@ -21,6 +21,7 @@
 using System;
 using System.Collections;
 using System.Collections.Generic;
+using System.Collections.ObjectModel;
 using System.IO;
 
 using MonoDevelop.Core.Gui.Components;
@@ -51,11 +52,10 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		PixbufList cat_imglist;
 
 		TreeStore catStore;
-		const string defaultSelectedCategory = "General";
 		
 		// Add To Project widgets
-		Combine solution;
 		string[] projectNames;
+		Project[] projectRefs;
 		
 		Project parentProject;
 		string basePath;
@@ -96,12 +96,6 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			}
 		}
 		
-		public override void Dispose ()
-		{
-			Destroy ();
-			base.Dispose ();
-		}
-		
 		void InitializeView()
 		{
 			PixbufList smalllist  = new PixbufList();
@@ -128,26 +122,79 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			InsertCategories(TreeIter.Zero, categories);
 			
 			//select the most recently selected category (with a few fallbacks)
-			string lastSelected = PropertyService.Get<string> ("Dialogs.NewFileDialog.LastSelectedCategory", defaultSelectedCategory);
-			TreeIter iterToSelect = FindIterMatchingString (catStore, lastSelected);
-			if (TreeIter.Zero.Equals (iterToSelect))
-				iterToSelect = FindIterMatchingString (catStore, defaultSelectedCategory);
-			if (TreeIter.Zero.Equals (iterToSelect))
-				catStore.GetIterFirst (out iterToSelect);
-			if (catStore.IterIsValid (iterToSelect))
+			string lastSelected = PropertyService.Get<string> (GetCategoryPropertyKey (parentProject), "General");
+			TreeIter iterToSelect;
+			if (FindCatIter (lastSelected, out iterToSelect)
+			    || FindCatIter ("Misc", out iterToSelect)
+			    || catStore.GetIterFirst (out iterToSelect)) {
 				catView.Selection.SelectIter (iterToSelect);
+			}
 		}
 		
-		TreeIter FindIterMatchingString (TreeModel model, string firstColStringMatch)
+		protected override void OnDestroyed ()
 		{
-			TreeIter iter;
-			model.GetIterFirst (out iter);
-			do {
-				if (((string)model.GetValue (iter, 0)) == firstColStringMatch) {
-					return iter;
+			if (catStore != null) {
+				catStore.Dispose ();
+				catStore = null;
+			}
+			if (cat_imglist != null) {
+				cat_imglist.Clear ();
+				cat_imglist = null;
+			}
+			base.OnDestroyed ();
+		}
+		
+		static string GetCategoryPropertyKey (Project proj)
+		{
+			string key = "Dialogs.NewFileDialog.LastSelectedCategory";
+			if (proj != null) {
+				key += "." + proj.ProjectType;
+				if (proj is DotNetProject)
+					key += "." + ((DotNetProject)proj).LanguageName;
+			}
+			return key;
+		}
+		
+		bool FindCatIter (string catPath, out TreeIter iter)
+		{
+			string[] cats = catPath.Split ('/');
+			iter = TreeIter.Zero;
+			
+			TreeIter nextIter;
+			if (!catStore.GetIterFirst (out nextIter))
+				return false;
+			
+			for (int i = 0; i < cats.Length; i++) {
+				if (FindCategoryAtCurrentLevel (cats[i], ref nextIter)) {
+					iter = nextIter;
+					if (i >= cats.Length - 1 || !catStore.IterChildren (out nextIter, nextIter))
+						return true;
 				}
-			} while (model.IterNext (ref iter));
-			return TreeIter.Zero;
+			}
+			return false;
+		}
+		
+		bool FindCategoryAtCurrentLevel (string category, ref TreeIter iter)
+		{
+			TreeIter trial = iter;
+			do {
+				string val = (string)catStore.GetValue (trial, 0);
+				if (val == category) {
+					iter = trial;
+					return true;
+				}
+			} while (catStore.IterNext (ref trial));
+			return false;
+		}
+		
+		string GetCatPath (TreeIter iter)
+		{
+			TreeIter currentIter = iter;
+			string path = (string) catStore.GetValue (currentIter, 0);
+			while (catStore.IterParent (out currentIter, currentIter)) {
+				path = ((string) catStore.GetValue (currentIter, 0)) + "/" + path;
+			}
+			return path;
 		}
 		
 		void InsertCategories(TreeIter node, List<Category> catarray)
@@ -385,7 +432,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			//PropertyService.Set("Dialogs.NewProjectDialog.LargeImages", ((RadioButton)ControlDictionary["largeIconsRadioButton"]).Checked);
 			TreeIter selectedIter;
 			if (catView.Selection.GetSelected (out selectedIter))
-				PropertyService.Set("Dialogs.NewFileDialog.LastSelectedCategory", (string)catStore.GetValue (selectedIter, 0));
+				PropertyService.Set (GetCategoryPropertyKey (parentProject), GetCatPath (selectedIter));
 			
 			if (iconView.CurrentlySelected != null && nameEntry.Text.Length > 0) {
 				TemplateItem titem = (TemplateItem) iconView.CurrentlySelected;
@@ -399,13 +446,16 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				}
 				
 				try {
-					if (!item.Create (project, path, titem.Language, nameEntry.Text))
+					if (!item.Create (project, project, path, titem.Language, nameEntry.Text))
 						return;
 				} catch (Exception ex) {
-					Services.MessageService.ShowError (ex);
+					MessageService.ShowException (ex);
 					return;
 				}
 
+				if (project != null)
+					IdeApp.ProjectOperations.Save (project);
+
 				if (OnOked != null)
 					OnOked (null, null);
 				Respond (Gtk.ResponseType.Ok);
@@ -504,8 +554,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		void AddToProjectComboChanged (object o, EventArgs e)
 		{
 			int which = projectAddCombo.Active;
-			string projectName = projectNames[which];
-			Project project = solution.FindProject (projectName);
+			Project project = projectRefs [which];
 			
 			if (project != null) {
 				if (basePath == null || basePath == String.Empty ||
@@ -547,12 +596,9 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			nameEntry.Changed += new EventHandler (NameChanged);
 			nameEntry.Activated += new EventHandler (OpenEvent);
 			
-			CombineEntryCollection projects = null;
-			if (parentProject == null) {
-				solution = IdeApp.ProjectOperations.CurrentOpenCombine;
-				if (solution != null)
-					projects = solution.GetAllProjects ();
-			}
+			ReadOnlyCollection<Project> projects = null;
+			if (parentProject == null)
+				projects = IdeApp.Workspace.GetAllProjects ();
 			
 			if (projects != null) {
 				Project curProject = IdeApp.ProjectOperations.CurrentSelectedProject;
@@ -562,23 +608,26 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				projectAddCheckbox.Toggled += new EventHandler (AddToProjectToggled);
 				
 				projectNames = new string [projects.Count];
+				projectRefs = new Project [projects.Count];
 				int i = 0;
 				
-				foreach (Project project in projects)
-					projectNames[i++] = project.Name;
+				bool singleSolution = IdeApp.Workspace.Items.Count == 1 && IdeApp.Workspace.Items [0] is Solution;
 				
-				Array.Sort (projectNames);
-				if (curProject != null) {
-					for (i = 0; i < projectNames.Length; i++) {
-						if (projectNames[i] == curProject.Name)
-							break;
-					}
+				foreach (Project project in projects) {
+					projectRefs[i] = project;
+					if (singleSolution)
+						projectNames[i++] = project.Name;
+					else
+						projectNames[i++] = project.ParentSolution.Name + "/" + project.Name;
 				}
 				
+				Array.Sort (projectNames, projectRefs);
+				i = Array.IndexOf (projectRefs, curProject);
+				
 				foreach (string pn in projectNames)
 					projectAddCombo.AppendText (pn);
 				
-				projectAddCombo.Active = i;
+				projectAddCombo.Active = i != -1 ? i : 0;
 				projectAddCombo.Sensitive = projectAddCheckbox.Active;
 				projectAddCombo.Changed += new EventHandler (AddToProjectComboChanged);
 				
