@@ -9,7 +9,7 @@ using MonoDevelop.Core.ProgressMonitoring;
 
 namespace MonoDevelop.Ide.Gui.Dialogs {
 	
-	public class SplashScreenForm : Gtk.Window, IProgressMonitor
+	public class SplashScreenForm : Gtk.Window, IProgressMonitor, IDisposable
 	{
 		static SplashScreenForm splashScreen;
 		static ProgressBar progress;
@@ -54,9 +54,17 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 		{
 			Gdk.GC gc = Style.LightGC (StateType.Normal);
 			GdkWindow.DrawPixbuf (gc, bitmap, 0, 0, 0, 0, bitmap.Width, bitmap.Height, Gdk.RgbDither.None, 0, 0);
+
+			Pango.Layout pl = new Pango.Layout (PangoContext);
+			Pango.FontDescription des = this.Style.FontDescription.Copy();
+			pl.FontDescription = des;
+		//	pl.SetMarkup("<b><span foreground='#cccccc'>" + BuildVariables.PackageVersionLabel + "</span></b>");
+			int w,h;
+			pl.GetPixelSize (out w, out h);
+			GdkWindow.DrawLayout (gc, bitmap.Width - w - 40, 110, pl);
 			return base.OnExposeEvent (evt);
 		}
-
+		
 		public static void SetProgress (double Percentage)
 		{
 			progress.Fraction = Percentage;
@@ -65,7 +73,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 
 		public static void SetMessage (string Message)
 		{
-			label.Markup = "<span size='small' foreground='#707070'>" + Message + "</span>";
+			label.Markup = "<span size='small' foreground='white'>" + Message + "</span>";
 			RunMainLoop ();
 		}
 		
@@ -135,10 +143,9 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 			get { return this; }
 		}
 		
-		public override void Dispose ()
+		void IDisposable.Dispose ()
 		{
 			Destroy ();
-			base.Dispose ();
 		}
 	}
 }
