@@ -36,17 +36,17 @@ using Gtk;
 
 namespace MonoDevelop.Ide.Gui.Pads
 {
-	internal class ErrorListPad : IPadContent, ILocationListPad
+	public class ErrorListPad : IPadContent, ILocationListPad
 	{
 		VBox control;
 		ScrolledWindow sw;
-		Gtk.TreeView view;
+		MonoDevelop.Ide.Gui.Components.PadTreeView view;
 		ListStore store;
 		TreeModelFilter filter;
 		ToggleToolButton errorBtn, warnBtn, msgBtn;
 		Gtk.Tooltips tips = new Gtk.Tooltips ();
 		Hashtable tasks = new Hashtable ();
-		IPadWindow window;
+//		IPadWindow window;
 		bool initializeLocation = true;
 
 		Menu menu;
@@ -69,6 +69,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 			Line,
 			Description,
 			File,
+			Project,
 			Path,
 			Task,
 			Read,
@@ -78,7 +79,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 
 		void IPadContent.Initialize (IPadWindow window)
 		{
-			this.window = window;
+//			this.window = window;
 			window.Title = GettextCatalog.GetString ("Error List");
 			window.Icon = MonoDevelop.Core.Gui.Stock.Error;
 		}
@@ -139,9 +140,10 @@ namespace MonoDevelop.Ide.Gui.Pads
 			
 			store = new Gtk.ListStore (typeof (Gdk.Pixbuf), // image - type
 			                           typeof (bool),       // marked?
-			                           typeof (int),        // line
+			                           typeof (string),        // line
 			                           typeof (string),     // desc
 			                           typeof (string),     // file
+			                           typeof (string),     // project
 			                           typeof (string),     // path
 			                           typeof (Task),       // task
 			                           typeof (bool),       // read?
@@ -156,7 +158,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 			filter = new TreeModelFilter (store, null);
             filter.VisibleFunc = filterFunct;
 			
-			view = new Gtk.TreeView (filter);
+			view = new MonoDevelop.Ide.Gui.Components.PadTreeView (filter);
 			view.RulesHint = true;
 			view.PopupMenu += new PopupMenuHandler (OnPopupMenu);
 			view.ButtonPressEvent += new ButtonPressEventHandler (OnButtonPressed);
@@ -168,12 +170,13 @@ namespace MonoDevelop.Ide.Gui.Pads
 			sw.ShadowType = ShadowType.None;
 			sw.Add (view);
 			
-			Services.TaskService.TasksCleared     += (EventHandler) DispatchService.GuiDispatch (new EventHandler (ShowResults));
+			Services.TaskService.TaskRemoved      += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (ShowResults));
 			Services.TaskService.TaskAdded        += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (TaskAdded));
 			Services.TaskService.TaskChanged      += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (TaskChanged));
 			
-			IdeApp.ProjectOperations.CombineOpened += (CombineEventHandler) DispatchService.GuiDispatch (new CombineEventHandler (OnCombineOpen));
-			IdeApp.ProjectOperations.CombineClosed += (CombineEventHandler) DispatchService.GuiDispatch (new CombineEventHandler (OnCombineClosed));
+			IdeApp.Workspace.FirstWorkspaceItemOpened += OnCombineOpen;
+			IdeApp.Workspace.LastWorkspaceItemClosed += OnCombineClosed;
+			
 			view.RowActivated            += new RowActivatedHandler (OnRowActivated);
 						
 			iconWarning = sw.RenderIcon (Gtk.Stock.DialogWarning, Gtk.IconSize.Menu, "");
@@ -192,15 +195,17 @@ namespace MonoDevelop.Ide.Gui.Pads
 			foreach (Task t in Services.TaskService.Tasks) {
 				AddTask (t);
 			}
+
+			control.FocusChain = new Gtk.Widget [] { sw };
 		}
 		
 		void LoadColumnsVisibility ()
 		{
-			string columns = (string)PropertyService.Get ("Monodevelop.ErrorListColumns", "TRUE;TRUE;TRUE;TRUE;TRUE;TRUE");
+			string columns = (string)PropertyService.Get ("Monodevelop.ErrorListColumns", "TRUE;TRUE;TRUE;TRUE;TRUE;TRUE;TRUE");
 			string[] tokens = columns.Split (new char[] {';'}, StringSplitOptions.RemoveEmptyEntries);
-			if (tokens.Length == 6 && view != null && view.Columns.Length == 6)
+			if (tokens.Length == 7 && view != null && view.Columns.Length == 7)
 			{
-				for (int i = 0; i < 6; i++)
+				for (int i = 0; i < 7; i++)
 				{
 					bool visible;
 					if (bool.TryParse (tokens[i], out visible))
@@ -211,12 +216,13 @@ namespace MonoDevelop.Ide.Gui.Pads
 
 		void StoreColumnsVisibility ()
 		{
-			string columns = String.Format ("{0};{1};{2};{3};{4};{5}",
+			string columns = String.Format ("{0};{1};{2};{3};{4};{5};{6}",
 			                                view.Columns[(int)Columns.Type].Visible,
 			                                view.Columns[(int)Columns.Marked].Visible,
 			                                view.Columns[(int)Columns.Line].Visible,
 			                                view.Columns[(int)Columns.Description].Visible,
 			                                view.Columns[(int)Columns.File].Visible,
+			                                view.Columns[(int)Columns.Project].Visible,
 			                                view.Columns[(int)Columns.Path].Visible);
 			PropertyService.Set ("Monodevelop.ErrorListColumns", columns);
 		}
@@ -275,6 +281,12 @@ namespace MonoDevelop.Ide.Gui.Pads
 				columnsActions[columnFile] = (int)Columns.File;
 				group.Add (columnFile);
 
+				ToggleAction columnProject = new ToggleAction ("columnProject", GettextCatalog.GetString ("Project"),
+				                                            GettextCatalog.GetString ("Toggle visibility of Project column"), null);
+				columnProject.Toggled += new EventHandler (OnColumnVisibilityChanged);
+				columnsActions[columnProject] = (int)Columns.Project;
+				group.Add (columnProject);
+
 				ToggleAction columnPath = new ToggleAction ("columnPath", GettextCatalog.GetString ("Path"),
 				                                            GettextCatalog.GetString ("Toggle visibility of Path column"), null);
 				columnPath.Toggled += new EventHandler (OnColumnVisibilityChanged);
@@ -295,6 +307,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 					+ "<menuitem action='columnLine' />"
 					+ "<menuitem action='columnDescription' />"
 					+ "<menuitem action='columnFile' />"
+					+ "<menuitem action='columnProject' />"
 					+ "<menuitem action='columnPath' />"
 					+ "</menu>"
 					+ "</popup></ui>";
@@ -310,6 +323,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 					columnLine.Active = view.Columns[(int)Columns.Line].Visible;
 					columnDescription.Active = view.Columns[(int)Columns.Description].Visible;
 					columnFile.Active = view.Columns[(int)Columns.File].Visible;
+					columnProject.Active = view.Columns[(int)Columns.Project].Visible;
 					columnPath.Active = view.Columns[(int)Columns.Path].Visible;
 					help.Sensitive = copy.Sensitive = jump.Sensitive =
 						view.Selection != null &&
@@ -342,11 +356,9 @@ namespace MonoDevelop.Ide.Gui.Pads
 			get {
 				TreeModel model;
 				TreeIter iter;
-				if (view.Selection.GetSelected (out model, out iter))
-				{
-					return (Task)model.GetValue (iter, (int)Columns.Task);
-				}
-				else return null; // no one selected
+				if (view.Selection.GetSelected (out model, out iter)) 
+					return model.GetValue (iter, (int)Columns.Task) as Task;
+				return null; // no one selected
 			}
 		}
 
@@ -388,9 +400,11 @@ namespace MonoDevelop.Ide.Gui.Pads
 			if (view.Selection.GetSelected (out model, out iter)) {
 				iter = filter.ConvertIterToChildIter (iter);
 				store.SetValue (iter, (int)Columns.Weight, (int) Pango.Weight.Normal);
-				Task task = (Task) store.GetValue (iter, (int)Columns.Task);
-				if (task != null)
+				Task task = store.GetValue (iter, (int)Columns.Task) as Task;
+				if (task != null) {
+					DisplayTask (task);
 					task.JumpToPosition ();
+				}
 			}
 		}
 
@@ -410,31 +424,30 @@ namespace MonoDevelop.Ide.Gui.Pads
 			
 			Gtk.CellRendererToggle toggleRender = new Gtk.CellRendererToggle ();
 			toggleRender.Toggled += new ToggledHandler (ItemToggled);
-
-			Gtk.CellRendererText line = new Gtk.CellRendererText (), desc = new Gtk.CellRendererText () , path = new Gtk.CellRendererText (),
-			file = new Gtk.CellRendererText ();
-
+			
 			TreeViewColumn col;
 			col = view.AppendColumn ("!", iconRender, "pixbuf", Columns.Type);
 			col.Clickable = true;
 			col.Clicked += new EventHandler (OnResortTasks);
 			col.SortIndicator = true;
 			view.AppendColumn ("", toggleRender, "active", Columns.Marked);
-			view.AppendColumn (GettextCatalog.GetString ("Line"), line, "text", Columns.Line, "weight", Columns.Weight);
-			col = view.AppendColumn (GettextCatalog.GetString ("Description"), desc, "text", Columns.Description, "weight", Columns.Weight, "strikethrough", Columns.Marked);
+			view.AppendColumn (GettextCatalog.GetString ("Line"), view.TextRenderer, "text", Columns.Line, "weight", Columns.Weight);
+			col = view.AppendColumn (GettextCatalog.GetString ("Description"), view.TextRenderer, "text", Columns.Description, "weight", Columns.Weight, "strikethrough", Columns.Marked);
 			col.Resizable = true;
-			col = view.AppendColumn (GettextCatalog.GetString ("File"), file, "text", Columns.File, "weight", Columns.Weight);
+			col = view.AppendColumn (GettextCatalog.GetString ("File"), view.TextRenderer, "text", Columns.File, "weight", Columns.Weight);
 			col.Resizable = true;
-			col = view.AppendColumn (GettextCatalog.GetString ("Path"), path, "text", Columns.Path, "weight", Columns.Weight);
+			col = view.AppendColumn (GettextCatalog.GetString ("Project"), view.TextRenderer, "text", Columns.Project, "weight", Columns.Weight);
+			col.Resizable = true;
+			col = view.AppendColumn (GettextCatalog.GetString ("Path"), view.TextRenderer, "text", Columns.Path, "weight", Columns.Weight);
 			col.Resizable = true;
 		}
 		
-		void OnCombineOpen(object sender, CombineEventArgs e)
+		void OnCombineOpen(object sender, EventArgs e)
 		{
 			Clear();
 		}
 		
-		void OnCombineClosed(object sender, CombineEventArgs e)
+		void OnCombineClosed(object sender, EventArgs e)
 		{
 			Clear();
 		}
@@ -443,20 +456,6 @@ namespace MonoDevelop.Ide.Gui.Pads
 		{
 		}
 		
-		void SelectTaskView (object s, BuildEventArgs args)
-		{
-			if (Services.TaskService.Tasks.Count > 0) {
-				try {
-					if (window.Visible)
-						window.Activate ();
-					else if ((bool) PropertyService.Get ("SharpDevelop.ShowTaskListAfterBuild", true)) {
-						window.Visible = true;
-						window.Activate ();
-					}
-				} catch {}
-			}
-		}
-		
 		void OnRowActivated (object o, RowActivatedArgs args)
 		{
 			OnTaskJumpto (null, null);
@@ -479,7 +478,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 			bool canShow = false;
 
 			try {
-				Task task = (Task) store.GetValue (iter, (int)Columns.Task);
+				Task task = store.GetValue (iter, (int)Columns.Task) as Task;
 				if (task == null)
 					return true;
 				if (task.TaskType == TaskType.Error && errorBtn.Active) canShow = true;
@@ -497,10 +496,8 @@ namespace MonoDevelop.Ide.Gui.Pads
 		{
 			Clear();
 
-			foreach (Task t in Services.TaskService.Tasks) {
+			foreach (Task t in Services.TaskService.Tasks)
 				AddTask (t);
-			}
-			SelectTaskView (null, null);
 		}
 
 		private void Clear()
@@ -519,17 +516,21 @@ namespace MonoDevelop.Ide.Gui.Pads
 			TreeIter iter;
 			if (store.GetIterFirst (out iter)) {
 				do {
-					Task curTask = (Task)store.GetValue (iter, 6);
+					Task curTask = store.GetValue (iter, (int)Columns.Task) as Task;
+					if (curTask == null) {
+						LoggingService.LogWarning ("Error list pad: Can't cast object: " + store.GetValue (iter, (int)Columns.Task) + " - it is not a Task.");
+						continue;
+					}
 					foreach (Task task in e.Tasks) {
 						if (task == curTask) {
-							store.SetValue (iter, 2, task.Line);
+							store.SetValue (iter, (int)Columns.Line, task.Line != 0 ? task.Line.ToString () : "");
 						}
 					}
 					
 				} while (store.IterNext (ref iter));
 			}
 		}
-		
+	
 		void TaskAdded (object sender, TaskEventArgs e)
 		{
 			AddTasks (e.Tasks);
@@ -596,8 +597,8 @@ namespace MonoDevelop.Ide.Gui.Pads
 			}
 			
 			string tmpPath = t.FileName;
-			if (t.Project != null)
-				tmpPath = FileService.AbsoluteToRelativePath (t.Project.BaseDirectory, t.FileName);
+			if (t.WorkspaceObject != null)
+				tmpPath = FileService.AbsoluteToRelativePath (t.WorkspaceObject.BaseDirectory, t.FileName);
 			
 			string fileName = tmpPath;
 			string path     = tmpPath;
@@ -610,15 +611,23 @@ namespace MonoDevelop.Ide.Gui.Pads
 				path = Path.GetDirectoryName(tmpPath);
 			} catch (Exception) {}
 			
+			string project;
+			if (t.OwnerItem is SolutionItem)
+				project = ((SolutionItem)t.OwnerItem).Name;
+			else
+				project = string.Empty;
+			
 			store.AppendValues (stock,
 			                    false,
-			                    t.Line,
+			                    t.Line != 0 ? t.Line.ToString () : "",
 			                    t.Description,
 			                    fileName,
+			                    project,
 			                    path,
 			                    t,
 			                    false,
-			                    (int) Pango.Weight.Bold);
+			                    (int) Pango.Weight.Bold
+			                    );
 		}
 
 		void UpdateErrorsNum () 
@@ -706,10 +715,19 @@ namespace MonoDevelop.Ide.Gui.Pads
 				line = 0;
 				column = 0;
 				view.Selection.UnselectAll ();
+				DisplayTask (null);
 				return false;
 			} else {
 				view.Selection.SelectIter (iter);
-				Task t = (Task) model.GetValue (iter, (int)Columns.Task);
+				Task t =  model.GetValue (iter, (int)Columns.Task) as Task;
+				if (t == null) {
+					file = null;
+					line = 0;
+					column = 0;
+					view.Selection.UnselectAll ();
+					DisplayTask (null);
+					return false;
+				}
 				file = t.FileName;
 				if (file == null)
 					return GetNextLocation (out file, out line, out column);
@@ -718,6 +736,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 				view.ScrollToCell (view.Model.GetPath (iter), view.Columns[0], false, 0, 0);
 				iter = filter.ConvertIterToChildIter (iter);
 				store.SetValue (iter, (int)Columns.Weight, (int) Pango.Weight.Normal);
+				DisplayTask (t);
 				return true;
 			}
 		}
@@ -726,7 +745,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 		{
 			bool hasNext, hasSel;
 			TreeIter iter;
-			TreeIter selIter;
+			TreeIter selIter = TreeIter.Zero;
 			TreeIter prevIter = TreeIter.Zero;
 			
 			hasSel = !initializeLocation && view.Selection.GetSelected (out selIter);
@@ -745,10 +764,19 @@ namespace MonoDevelop.Ide.Gui.Pads
 				line = 0;
 				column = 0;
 				view.Selection.UnselectAll ();
+				DisplayTask (null);
 				return false;
 			} else {
 				view.Selection.SelectIter (prevIter);
-				Task t = (Task) view.Model.GetValue (prevIter, (int)Columns.Task);
+				Task t = view.Model.GetValue (prevIter, (int)Columns.Task) as Task;
+				if (t == null) {
+					file = null;
+					line = 0;
+					column = 0;
+					view.Selection.UnselectAll ();
+					DisplayTask (null);
+					return false;
+				}
 				file = t.FileName;
 				if (file == null)
 					return GetPreviousLocation (out file, out line, out column);
@@ -757,8 +785,21 @@ namespace MonoDevelop.Ide.Gui.Pads
 				view.ScrollToCell (view.Model.GetPath (prevIter), view.Columns[0], false, 0, 0);
 				prevIter = filter.ConvertIterToChildIter (prevIter);
 				store.SetValue (prevIter, (int)Columns.Weight, (int) Pango.Weight.Normal);
+				DisplayTask (t);
 				return true;
 			}
 		}
+
+		void DisplayTask (Task t)
+		{
+			if (t == null)
+				IdeApp.Workbench.StatusBar.ShowMessage (GettextCatalog.GetString ("No more errors or warnings"));
+			else if (t.TaskType == TaskType.Error)
+				IdeApp.Workbench.StatusBar.ShowError (t.ErrorNumber + " - " + t.Description);
+			else if (t.TaskType == TaskType.Warning)
+				IdeApp.Workbench.StatusBar.ShowError (t.ErrorNumber + " - " + t.Description);
+			else
+				IdeApp.Workbench.StatusBar.ShowMessage (t.Description);
+		}
 	}
 }
