@@ -36,9 +36,11 @@ using MonoDevelop.Core.Gui;
 using MonoDevelop.Core.Gui.Components;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Core.Gui.Dialogs;
-
+using MonoDevelop.Ide.Commands;
 using MonoDevelop.Ide.Codons;
 using MonoDevelop.Ide.Gui.Dialogs;
+using MonoDevelop.Projects.Dom.Parser;
+using MonoDevelop.Components.Commands;
 
 using GLib;
 
@@ -51,7 +53,7 @@ namespace MonoDevelop.Ide.Gui
 	{
 		readonly static string mainMenuPath    = "/MonoDevelop/Ide/MainMenu";
 		readonly static string viewContentPath = "/MonoDevelop/Ide/Pads";
-		readonly static string toolbarsPath = "/MonoDevelop/Ide/Toolbar";
+		readonly static string toolbarsPath    = "/MonoDevelop/Ide/Toolbar";
 		
 		List<PadCodon> padContentCollection      = new List<PadCodon> ();
 		List<IViewContent> viewContentCollection = new List<IViewContent> ();
@@ -65,8 +67,15 @@ namespace MonoDevelop.Ide.Gui
 
 		internal static GType gtype;
 		
-		public Gtk.MenuBar TopMenu = null;
+		Gtk.MenuBar topMenu = null;
 		private Gtk.Toolbar[] toolbars = null;
+		MonoDevelopStatusBar statusBar = new MonoDevelop.Ide.MonoDevelopStatusBar ();
+		
+		public MonoDevelopStatusBar StatusBar {
+			get {
+				return statusBar;
+			}
+		}
 		
 		enum TargetList {
 			UriList = 100
@@ -76,6 +85,10 @@ namespace MonoDevelop.Ide.Gui
 			new Gtk.TargetEntry ("text/uri-list", 0, (uint)TargetList.UriList)
 		};
 		
+		public Gtk.MenuBar TopMenu {
+			get { return topMenu; }
+		}
+		
 		public bool FullScreen {
 			get {
 				return fullscreen;
@@ -188,8 +201,8 @@ namespace MonoDevelop.Ide.Gui
 					file = new Uri(file).LocalPath;
 
 					try {
-						if (Services.ProjectService.IsCombineEntryFile (file))
-							IdeApp.ProjectOperations.OpenCombine(file);
+						if (Services.ProjectService.IsWorkspaceItemFile (file))
+							IdeApp.Workspace.OpenWorkspaceItem(file);
 						else
 							IdeApp.Workbench.OpenDocument (file);
 					} catch (Exception e) {
@@ -210,7 +223,7 @@ namespace MonoDevelop.Ide.Gui
 //			TopMenu.Selected   += new CommandHandler(OnTopMenuSelected);
 //			TopMenu.Deselected += new CommandHandler(OnTopMenuDeselected);
 
-			TopMenu = IdeApp.CommandService.CreateMenuBar (mainMenuPath);
+			topMenu = IdeApp.CommandService.CreateMenuBar (mainMenuPath);
 			toolbars = IdeApp.CommandService.CreateToolbarSet (toolbarsPath);
 			foreach (Gtk.Toolbar t in toolbars) {
 				t.ToolbarStyle = Gtk.ToolbarStyle.Icons;
@@ -232,7 +245,7 @@ namespace MonoDevelop.Ide.Gui
 			bool changed = false;
 			
 			if (args.PathChanged (mainMenuPath)) {
-				TopMenu = IdeApp.CommandService.CreateMenuBar (mainMenuPath);
+				topMenu = IdeApp.CommandService.CreateMenuBar (mainMenuPath);
 				changed = true;
 			}
 			
@@ -247,11 +260,8 @@ namespace MonoDevelop.Ide.Gui
 				layout.RedrawAllComponents();
 		}
 				
-		public void CloseContent(IViewContent content)
+		public void CloseContent (IViewContent content)
 		{
-			if (PropertyService.Get("SharpDevelop.LoadDocumentProperties", true) && content is IMementoCapable) {
-				StoreMemento(content);
-			}
 			if (viewContentCollection.Contains(content)) {
 				viewContentCollection.Remove(content);
 			}
@@ -291,24 +301,82 @@ namespace MonoDevelop.Ide.Gui
 			
 			if (bringToFront)
 				content.WorkbenchWindow.SelectWindow();
-
+			
 			RedrawAllComponents ();
 			
 			IEditableTextBuffer editor = (IEditableTextBuffer) content.GetContent (typeof(IEditableTextBuffer));
 			if (editor != null)
-				editor.TextChanged += new TextChangedEventHandler (OnViewTextChanged);
+				editor.TextChanged += OnViewTextChanged;
 		}
 		
-		public virtual void ShowPad (PadCodon content)
+		void ShowPadNode (ExtensionNode node)
 		{
-			padContentCollection.Add(content);
+			if (node is PadCodon) {
+				PadCodon pad = (PadCodon) node;
+				ShowPad (pad);
+				if (layout != null) {
+					IPadWindow win = WorkbenchLayout.GetPadWindow (pad);
+					if (pad.Label != null)
+						win.Title = pad.Label;
+					if (pad.Icon != null)
+						win.Icon = pad.Icon;
+				}
+			}
+			else if (node is CategoryNode) {
+				foreach (ExtensionNode cn in node.ChildNodes)
+					ShowPadNode (cn);
+			}
+		}
+		
+		void RemovePadNode (ExtensionNode node)
+		{
+			if (node is PadCodon)
+				RemovePad ((PadCodon) node);
+			else if (node is CategoryNode) {
+				foreach (ExtensionNode cn in node.ChildNodes)
+					RemovePadNode (cn);
+			}
+		}
+		
+		public void ShowPad (PadCodon content)
+		{
+			AddPad (content, true);
+		}
+		
+		public void AddPad (PadCodon content)
+		{
+			AddPad (content, false);
+		}
+		
+		void AddPad (PadCodon content, bool show)
+		{
+			if (padContentCollection.Contains (content))
+				return;
+
+			if (content.HasId) {
+				ActionCommand cmd = new ActionCommand ("Pad|" + content.PadId, GettextCatalog.GetString (content.Label), null);
+				cmd.DefaultHandler = new PadActivationHandler (this, content);
+				cmd.Category = GettextCatalog.GetString ("View");
+				cmd.Description = GettextCatalog.GetString ("Show {0}", cmd.Text);
+				IdeApp.CommandService.RegisterCommand (cmd);
+			}
+			padContentCollection.Add (content);
 			
-			if (layout != null)
-				layout.ShowPad (content);
+			if (layout != null) {
+				if (show)
+					layout.ShowPad (content);
+				else
+					layout.AddPad (content);
+			}
 		}
 		
 		public void RemovePad (PadCodon codon)
 		{
+			if (codon.HasId) {
+				Command cmd = IdeApp.CommandService.GetCommand (codon.Id);
+				if (cmd != null)
+					IdeApp.CommandService.UnregisterCommand (cmd);
+			}
 			padContentCollection.Remove (codon);
 			
 			if (layout != null)
@@ -347,44 +415,13 @@ namespace MonoDevelop.Ide.Gui
 				string fileName = content.ContentName.Substring(3).Replace('/', '.').Replace('\\', '.').Replace(System.IO.Path.DirectorySeparatorChar, '.');
 				string fullFileName = directory + System.IO.Path.DirectorySeparatorChar + fileName;
 				// check the file name length because it could be more than the maximum length of a file name
-				if (FileService.IsValidFileName(fullFileName) && File.Exists(fullFileName)) {
+				if (FileService.IsValidPath(fullFileName) && File.Exists(fullFileName)) {
 					return Properties.Load (fullFileName);
 				}
 			}
 			return null;
 		}
 		
-		public void StoreMemento(IViewContent content)
-		{
-			if (content.ContentName == null) {
-				return;
-			}
-			string directory = System.IO.Path.Combine (PropertyService.ConfigPath, "temp");
-			if (!Directory.Exists(directory)) {
-				Directory.CreateDirectory(directory);
-			}
-			
-//			XmlDocument doc = new XmlDocument();
-//			doc.LoadXml("<?xml version=\"1.0\"?>\n<Mementoable/>");
-//			
-//			XmlAttribute fileAttribute = doc.CreateAttribute("file");
-//			fileAttribute.InnerText = content.ContentName;
-//			doc.DocumentElement.Attributes.Append(fileAttribute);
-//			
-			object memento = ((IMementoCapable)content).CreateMemento();
-			
-//			doc.DocumentElement.AppendChild(memento.ToXmlElement(doc));
-//			
-			string fileName = content.ContentName.Substring(3).Replace('/', '.').Replace('\\', '.').Replace(System.IO.Path.DirectorySeparatorChar, '.');
-			// check the file name length because it could be more than the maximum length of a file name
-			string fullFileName = System.IO.Path.Combine (directory, fileName);
-			if (FileService.IsValidFileName(fullFileName)) {
-				((Properties)memento).Save (fullFileName);
-//				doc.Save (fullFileName);
-			}
-			
-		}
-		
 		// interface IMementoCapable
 		public ICustomXmlSerializer CreateMemento()
 		{
@@ -491,7 +528,7 @@ namespace MonoDevelop.Ide.Gui
 			if (!IdeApp.OnExit ())
 				return false;
 
-			IdeApp.ProjectOperations.SaveCombinePreferences ();
+			IdeApp.Workspace.SavePreferences ();
 			IdeApp.CommandService.Dispose ();
 
 			bool showDirtyDialog = false;
@@ -513,7 +550,7 @@ namespace MonoDevelop.Ide.Gui
 			
 			CloseAllViews ();
 			
-			IdeApp.ProjectOperations.CloseCombine (false);
+			IdeApp.Workspace.Close (false);
 			PropertyService.Set ("SharpDevelop.Workbench.WorkbenchMemento", CreateMemento());
 			IdeApp.OnExited ();
 			OnClosed (null);
@@ -536,16 +573,17 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
-		bool parsingFile;
+//		bool parsingFile;
 		
 		void OnViewTextChanged (object sender, TextChangedEventArgs e)
 		{
-			if (!parsingFile) {
+//document does it.
+/*			if (!parsingFile) {
 				parsingFile = true;
 				GLib.Timeout.Add (500, new TimeoutHandler (ParseCurrentFile));
-			}
+			}*/
 		}
-		
+		/*
 		bool ParseCurrentFile ()
 		{
 			parsingFile = false;
@@ -571,7 +609,7 @@ namespace MonoDevelop.Ide.Gui
 			if (fileName == null || fileName.Length == 0)
 				return false;
 			
-			if (Services.ParserService.GetParser (fileName) == null)
+			if (ProjectDomService.GetParserByFileName (fileName) == null)
 				return false;
 			
 			string text = editable.Text;
@@ -586,9 +624,9 @@ namespace MonoDevelop.Ide.Gui
 		void AsyncParseCurrentFile (object ob)
 		{
 			object[] data = (object[]) ob;
-			IdeApp.ProjectOperations.ParserDatabase.UpdateFile ((Project) data[0], (string) data[1], (string) data[2]);
+			IdeApp.Workspace.ParserDatabase.UpdateFile ((Project) data[0], (string) data[1], (string) data[2]);
 		}
-
+*/
 		public Gtk.Toolbar[] ToolBars {
 			get { return toolbars; }
 		}
@@ -615,22 +653,17 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void InitializeLayout (IWorkbenchLayout workbenchLayout)
 		{
-			ExtensionNodeList padCodons = AddinManager.GetExtensionNodes (viewContentPath, typeof(PadCodon));
+			ExtensionNodeList padCodons = AddinManager.GetExtensionNodes (viewContentPath);
 			
-			foreach (PadCodon codon in padCodons)
-				ShowPad (codon);
+			foreach (ExtensionNode node in padCodons)
+				ShowPadNode (node);
 			
 			layout = workbenchLayout;
 			layout.Attach(this);
 			layout.ActiveWorkbenchWindowChanged += windowChangeEventHandler;
 			
-			foreach (PadCodon codon in padCodons) {
-				IPadWindow win = WorkbenchLayout.GetPadWindow (codon);
-				if (codon.Label != null)
-					win.Title = codon.Label;
-				if (codon.Icon != null)
-					win.Icon = codon.Icon;
-			}
+			foreach (ExtensionNode node in padCodons)
+				ShowPadNode (node);
 
 			RedrawAllComponents ();
 			
@@ -640,6 +673,83 @@ namespace MonoDevelop.Ide.Gui
 			initializing = false;
 		}
 		
+		public void ResetToolbars ()
+		{
+			layout.ResetToolbars ();
+		}
+
+		protected override bool OnKeyPressEvent (Gdk.EventKey evnt)
+		{
+			// Handle Tab+Control == NextWindow, Tab+Shift+Control == PrevWindow commands.
+			if (evnt.Key == Gdk.Key.Tab || evnt.Key == Gdk.Key.ISO_Left_Tab) {
+				if ((evnt.State & Gdk.ModifierType.ControlMask) == Gdk.ModifierType.ControlMask) {
+					bool selectNext = (evnt.State & Gdk.ModifierType.ShiftMask) != Gdk.ModifierType.ShiftMask;
+					if (PropertyService.Get ("MonoDevelop.Core.Gui.EnableDocumentSwitchDialog", true)) {
+						DocumentSwitcher switcher = new DocumentSwitcher (this, selectNext);
+						switcher.ShowAll ();
+						switcher.GrabFocus ();
+					} else {
+						IdeApp.CommandService.DispatchCommand (selectNext ? WindowCommands.NextWindow : WindowCommands.PrevWindow);
+					}
+					return true;
+				}
+			}
+			
+			// Handle Alt+1-0 keys
+			if ((evnt.State & Gdk.ModifierType.Mod1Mask) == Gdk.ModifierType.Mod1Mask) {		
+				switch (evnt.Key) {
+				case Gdk.Key.KP_1:
+				case Gdk.Key.Key_1:
+					SwitchToDocument (0);
+					return true;
+				case Gdk.Key.KP_2:
+				case Gdk.Key.Key_2:
+					SwitchToDocument (1);
+					return true;
+				case Gdk.Key.KP_3:
+				case Gdk.Key.Key_3:
+					SwitchToDocument (2);
+					return true;
+				case Gdk.Key.KP_4:
+				case Gdk.Key.Key_4:
+					SwitchToDocument (3);
+					return true;
+				case Gdk.Key.KP_5:
+				case Gdk.Key.Key_5:
+					SwitchToDocument (4);
+					return true;
+				case Gdk.Key.KP_6:
+				case Gdk.Key.Key_6:
+					SwitchToDocument (5);
+					return true;
+				case Gdk.Key.KP_7:
+				case Gdk.Key.Key_7:
+					SwitchToDocument (6);
+					return true;
+				case Gdk.Key.KP_8:
+				case Gdk.Key.Key_8:
+					SwitchToDocument (7);
+					return true;
+				case Gdk.Key.KP_9:
+				case Gdk.Key.Key_9:
+					SwitchToDocument (8);
+					return true;
+				case Gdk.Key.KP_0:
+				case Gdk.Key.Key_0:
+					SwitchToDocument (9);
+					return true;
+				}
+			}
+			return base.OnKeyPressEvent (evnt); 
+		}
+		
+		void SwitchToDocument (int number)
+		{
+			if (number >= viewContentCollection.Count || number < 0)
+				return;
+			viewContentCollection[number].WorkbenchWindow.SelectWindow ();
+		}
+		
 		bool initializing;
 		
 		void OnExtensionChanged (object s, ExtensionNodeEventArgs args)
@@ -648,20 +758,11 @@ namespace MonoDevelop.Ide.Gui
 				return;
 			
 			if (args.Change == ExtensionChange.Add) {
-				PadCodon codon = (PadCodon) args.ExtensionNode;
-				ShowPad (codon);
-				
-				IPadWindow win = WorkbenchLayout.GetPadWindow (codon);
-				if (codon.Label != null)
-					win.Title = codon.Label;
-				if (codon.Icon != null)
-					win.Icon = codon.Icon;
-
+				ShowPadNode (args.ExtensionNode);
 				RedrawAllComponents ();
 			}
 			else {
-				PadCodon codon = (PadCodon) args.ExtensionNode;
-				RemovePad (codon);
+				RemovePadNode (args.ExtensionNode);
 			}
 		}
 		
@@ -684,5 +785,22 @@ namespace MonoDevelop.Ide.Gui
 
 		public event EventHandler ContextChanged;
 	}
+
+	class PadActivationHandler: CommandHandler
+	{
+		PadCodon pad;
+		DefaultWorkbench wb;
+		
+		public PadActivationHandler (DefaultWorkbench wb, PadCodon pad)
+		{
+			this.pad = pad;
+			this.wb = wb;
+		}
+		
+		protected override void Run ()
+		{
+			wb.BringToFront (pad);
+		}
+	}
 }
 
