@@ -46,10 +46,15 @@ namespace MonoDevelop.Ide.Gui
 		ArrayList outputMonitors = new ArrayList ();
 		
 		/******************************/
+
+		internal void Initialize ()
+		{
+			CreateMonitorPad (GettextCatalog.GetString ("Build Output"), MonoDevelop.Core.Gui.Stock.BuildCombine, false, true, false);
+		}
 		
 		public IProgressMonitor GetBuildProgressMonitor ()
 		{
-			bool front = (bool) PropertyService.Get ("SharpDevelop.ShowOutputWindowAtBuild", true);
+			bool front = IdeApp.Preferences.ShowOutputPadWhenBuildStarts;
 			AggregatedProgressMonitor mon = new AggregatedProgressMonitor (GetOutputProgressMonitor (GettextCatalog.GetString ("Build Output"), MonoDevelop.Core.Gui.Stock.BuildCombine, front, true));
 			mon.AddSlaveMonitor (GetStatusProgressMonitor (GettextCatalog.GetString ("Building..."), MonoDevelop.Core.Gui.Stock.BuildCombine, false));
 			return mon;
@@ -94,6 +99,12 @@ namespace MonoDevelop.Ide.Gui
 		}
 		
 		public IProgressMonitor GetOutputProgressMonitor (string title, string icon, bool bringToFront, bool allowMonitorReuse)
+		{
+			DefaultMonitorPad monitorPad = CreateMonitorPad (title, icon, bringToFront, allowMonitorReuse, true);
+			return new OutputProgressMonitor (monitorPad, title, icon);
+		}
+		
+		DefaultMonitorPad CreateMonitorPad (string title, string icon, bool bringToFront, bool allowMonitorReuse, bool show)
 		{
 			Pad pad = null;
 			if (icon == null)
@@ -118,7 +129,7 @@ namespace MonoDevelop.Ide.Gui
 				}
 				if (pad != null) {
 					if (bringToFront) pad.BringToFront ();
-					return new OutputProgressMonitor ((DefaultMonitorPad) pad.Content, pad.Title, icon);
+					return (DefaultMonitorPad) pad.Content;
 				}
 			}
 
@@ -128,10 +139,18 @@ namespace MonoDevelop.Ide.Gui
 			string newPadId = "OutputPad - " + title + " - " + instanceCount;
 			string basePadId = "OutputPad - " + title + " - 0";
 			
-			if (instanceCount > 0)
+			if (instanceCount > 0) {
+				// Translate the title before adding the count
+				title = GettextCatalog.GetString (title);
 				title += " (" + (instanceCount+1) + ")";
+			}
+
+			if (show)
+				pad = IdeApp.Workbench.ShowPad (monitorPad, newPadId, title, basePadId + "/Center Bottom", icon);
+			else
+				pad = IdeApp.Workbench.AddPad (monitorPad, newPadId, title, basePadId + "/Center Bottom", icon);
 			
-			pad = IdeApp.Workbench.ShowPad (monitorPad, newPadId, title, basePadId + "/Center Bottom", icon);
+			pad.Sticky = true;
 			outputMonitors.Add (pad);
 			
 			if (instanceCount > 0) {
@@ -144,9 +163,9 @@ namespace MonoDevelop.Ide.Gui
 			if (bringToFront)
 				pad.BringToFront ();
 
-			return new OutputProgressMonitor (monitorPad, title, icon);
+			return monitorPad;
 		}
-
+		
 		public ISearchProgressMonitor GetSearchProgressMonitor (bool bringToFront)
 		{
 			Pad pad = null;
@@ -181,6 +200,7 @@ namespace MonoDevelop.Ide.Gui
 			
 			SearchResultPad monitorPad = new SearchResultPad (instanceNum);
 			pad = IdeApp.Workbench.ShowPad (monitorPad, newPadId, title, basePadId + "/Center Bottom", MonoDevelop.Core.Gui.Stock.FindIcon);
+			pad.Sticky = true;
 		    searchMonitors.Add (pad);
 
 			if (searchMonitors.Count > 1) {
