@@ -29,6 +29,7 @@
 
 using System;
 using System.Collections;
+using System.Diagnostics;
 using System.IO;
 using Gtk;
 
@@ -44,7 +45,7 @@ using MonoDevelop.Ide.Gui.Dialogs;
 
 namespace MonoDevelop.Ide.Gui
 {
-	public class ViewCommandHandlers: ICommandRouter
+	public class ViewCommandHandlers : ICommandRouter
 	{
 		IWorkbenchWindow window;
 		Document doc;
@@ -111,7 +112,9 @@ namespace MonoDevelop.Ide.Gui
 		[CommandHandler (FileCommands.ReloadFile)]
 		protected void OnReloadFile ()
 		{
-			if (Services.MessageService.AskQuestion(GettextCatalog.GetString ("Are you sure that you want to reload the file?")))
+			if (MessageService.GenericAlert (MonoDevelop.Core.Gui.Stock.Warning,
+			                                GettextCatalog.GetString ("Revert unsaved changes to document '{0}'?", Path.GetFileName (doc.Name)),
+			                                GettextCatalog.GetString ("All changes made to the document will be permanently lost."), 0, AlertButton.Cancel, AlertButton.Revert) == AlertButton.Revert)
 				doc.Reload ();
 		}
 		
@@ -158,17 +161,17 @@ namespace MonoDevelop.Ide.Gui
 		[CommandHandler (EditCommands.Cut)]
 		protected void OnCut ()
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			if (editable != null)
-				editable.ClipboardHandler.Cut(null, null);
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				handler.Cut ();
 		}
 		
 		[CommandUpdateHandler (EditCommands.Cut)]
 		protected void OnUpdateCut (CommandInfo info)
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			if (editable != null)
-				info.Enabled = editable.ClipboardHandler.EnableCut;
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				info.Enabled = handler.EnableCut;
 			else
 				info.Bypass = true;
 		}
@@ -176,17 +179,17 @@ namespace MonoDevelop.Ide.Gui
 		[CommandHandler (EditCommands.Copy)]
 		protected void OnCopy ()
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			if (editable != null)
-				editable.ClipboardHandler.Copy(null, null);
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				handler.Copy ();
 		}
 		
 		[CommandUpdateHandler (EditCommands.Copy)]
 		protected void OnUpdateCopy (CommandInfo info)
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			if (editable != null)
-				info.Enabled = editable.ClipboardHandler.EnableCopy;
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				info.Enabled = handler.EnableCopy;
 			else
 				info.Bypass = true;
 		}
@@ -194,118 +197,57 @@ namespace MonoDevelop.Ide.Gui
 		[CommandHandler (EditCommands.Paste)]
 		protected void OnPaste ()
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			if (editable != null)
-				editable.ClipboardHandler.Paste(null, null);
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				handler.Paste ();
 		}
 		
 		[CommandUpdateHandler (EditCommands.Paste)]
 		protected void OnUpdatePaste (CommandInfo info)
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			if (editable != null)
-				info.Enabled = editable.ClipboardHandler.EnablePaste;
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				info.Enabled = handler.EnablePaste;
 			else
 				info.Bypass = true;
 		}
 		
-/*		[CommandHandler (EditCommands.Delete)]
+		[CommandHandler (EditCommands.Delete)]
 		protected void OnDelete ()
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			if (editable != null)
-				editable.ClipboardHandler.Delete(null, null);
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				handler.Delete ();
 		}
 		
 		[CommandUpdateHandler (EditCommands.Delete)]
 		protected void OnUpdateDelete (CommandInfo info)
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			info.Enabled = editable != null && editable.ClipboardHandler.EnableDelete;
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				info.Enabled = handler.EnableDelete;
+			else
+				info.Bypass = true;
 		}
-*/		
+		
 		[CommandHandler (EditCommands.SelectAll)]
 		protected void OnSelectAll ()
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			if (editable != null)
-				editable.ClipboardHandler.SelectAll(null, null);
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				handler.SelectAll ();
 		}
 		
 		[CommandUpdateHandler (EditCommands.SelectAll)]
 		protected void OnUpdateSelectAll (CommandInfo info)
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
-			if (editable != null)
-				info.Enabled = editable.ClipboardHandler.EnableSelectAll;
+			IClipboardHandler handler = GetContent <IClipboardHandler> ();
+			if (handler != null)
+				info.Enabled = handler.EnableSelectAll;
 			else
 				info.Bypass = true;
 		}
 		
-		[CommandHandler (EditCommands.WordCount)]
-		protected void OnWordCount()
-		{
-			WordCountDialog wcd = new WordCountDialog ();
-			wcd.Run ();
-			wcd.Hide ();
-		}
-		
-		[CommandHandler (EditCommands.CommentCode)]
-		public void OnCommentCode()
-		{
-			ICodeStyleOperations  styling = GetContent<ICodeStyleOperations> ();
-			if (styling != null)
-				styling.CommentCode ();
-		}
-		
-		[CommandUpdateHandler (EditCommands.CommentCode)]
-		protected void OnUpdateCommentCode (CommandInfo info)
-		{
-			info.Enabled = GetContent<ICodeStyleOperations> () != null;
-		}
-		
-		[CommandHandler (EditCommands.UncommentCode)]
-		public void OnUncommentCode()
-		{
-			ICodeStyleOperations  styling = GetContent <ICodeStyleOperations> ();
-			if (styling != null)
-				styling.UncommentCode ();
-		}
-		
-		[CommandUpdateHandler (EditCommands.UncommentCode)]
-		protected void OnUpdateUncommentCode (CommandInfo info)
-		{
-			info.Enabled = GetContent<ICodeStyleOperations> () != null;
-		}
-		
-		[CommandHandler (EditCommands.IndentSelection)]
-		public void OnIndentSelection()
-		{
-			ICodeStyleOperations  styling = GetContent<ICodeStyleOperations> ();
-			if (styling != null)
-				styling.IndentSelection ();
-		}
-		
-		[CommandUpdateHandler (EditCommands.IndentSelection)]
-		protected void OnUpdateIndentSelection (CommandInfo info)
-		{
-			info.Enabled = GetContent<ICodeStyleOperations> () != null;
-		}
-		
-		[CommandHandler (EditCommands.UnIndentSelection)]
-		public void OnUnIndentSelection()
-		{
-			ICodeStyleOperations  styling = GetContent<ICodeStyleOperations> ();
-			if (styling != null)
-				styling.UnIndentSelection ();
-		}
-		
-		[CommandUpdateHandler (EditCommands.UnIndentSelection)]
-		protected void OnUpdateUnIndentSelection (CommandInfo info)
-		{
-			info.Enabled = GetContent<ICodeStyleOperations> () != null;
-		}
-		
 		[CommandHandler (EditCommands.UppercaseSelection)]
 		public void OnUppercaseSelection ()
 		{
@@ -328,12 +270,58 @@ namespace MonoDevelop.Ide.Gui
 				}
 			}
 		}
-		
-		[CommandUpdateHandler (EditCommands.UnIndentSelection)]
-		protected void OnUppercaseSelection (CommandInfo info)
+
+		[CommandHandler (EditCommands.JoinWithNextLine)]
+		public void OnJoinWithNextLine ()
 		{
-			info.Enabled = GetContent <IEditableTextBuffer> () != null;
+			IEditableTextBuffer buffer = GetContent <IEditableTextBuffer> ();
+			if (buffer != null)
+			{
+				int cursor_pos = buffer.CursorPosition;
+				int line, column;
+				buffer.GetLineColumnFromPosition (buffer.CursorPosition, out line, out column);
+
+				int start_pos = buffer.GetPositionFromLineColumn (line, 0)+1;
+				
+				int line_len = doc.TextEditor.GetLineLength (line);
+				int next_line_len = doc.TextEditor.GetLineLength (line+1);
+				
+				if (next_line_len <= 0) {
+					return;
+				}
+				
+				int end_pos = start_pos + line_len + next_line_len + 1;
+				
+				string curr_line = doc.TextEditor.GetLineText (line);
+				string next_line = doc.TextEditor.GetLineText (line+1);
+
+				string new_text = curr_line;
+				string next_line_trimmed = next_line.TrimStart ('\n', '\r', '\t', ' ');
+				if (next_line_trimmed != null &&
+				    next_line_trimmed != String.Empty) {
+					new_text += " " + next_line_trimmed;
+				}
+				
+				buffer.BeginAtomicUndo ();
+				buffer.DeleteText (start_pos, end_pos-start_pos);
+				buffer.InsertText (start_pos, new_text);
+				buffer.EndAtomicUndo ();
+
+				buffer.CursorPosition = cursor_pos;
+			}
 		}
+		/*
+		[CommandUpdateHandler (SearchCommands.GotoLineNumber)]
+		void OnUpdateGotoLineNumber (CommandInfo info)
+		{
+			info.Enabled = GetContent <IEditableTextBuffer> () != null && MonoDevelop.Ide.Gui.Dialogs.GotoLineDialog.CanShow;
+		}
+		
+		[CommandHandler (SearchCommands.GotoLineNumber)]
+		void OnGotoLineNumber ()
+		{
+			MonoDevelop.Ide.Gui.Dialogs.GotoLineDialog.ShowDialog (GetContent <IEditableTextBuffer> ());
+		}*/
 		
 		[CommandHandler (EditCommands.LowercaseSelection)]
 		public void OnLowercaseSelection ()
@@ -379,8 +367,9 @@ namespace MonoDevelop.Ide.Gui
 		[CommandUpdateHandler (TextEditorCommands.DocumentEnd)]
 		[CommandUpdateHandler (TextEditorCommands.DeleteLine)]
 		[CommandUpdateHandler (TextEditorCommands.DeleteToLineEnd)]
-		[CommandUpdateHandler (TextEditorCommands.MoveBlockUp)]		
+		[CommandUpdateHandler (TextEditorCommands.MoveBlockUp)]
 		[CommandUpdateHandler (TextEditorCommands.MoveBlockDown)]		
+		[CommandUpdateHandler (TextEditorCommands.GotoMatchingBrace)]		
 		protected void OnUpdateLineEnd (CommandInfo info)
 		{
 			// If the current document is not an editor, just ignore the text
@@ -463,6 +452,12 @@ namespace MonoDevelop.Ide.Gui
 			doc.TextEditor.CursorPosition = doc.TextEditor.TextLength;
 		}
 		
+		[CommandHandler (TextEditorCommands.GotoMatchingBrace)]
+		protected void OnGotoMatchingBrace ()
+		{
+			doc.TextEditor.GotoMatchingBrace ();
+		}
+		
 		[CommandHandler (TextEditorCommands.DeleteLine)]
 		protected void OnDeleteLine ()
 		{
@@ -566,5 +561,84 @@ namespace MonoDevelop.Ide.Gui
 			doc.TextEditor.Select(selStart, selEnd);
 			doc.TextEditor.EndAtomicUndo ();
 		}
+		
+		#region Folding
+		[CommandUpdateHandler (EditCommands.ToggleAllFoldings)]
+		[CommandUpdateHandler (EditCommands.FoldDefinitions)]
+		protected void UpdateFoldCommands (CommandInfo info)
+		{
+			info.Enabled = GetContent <IFoldable> () != null;
+		}
+		
+		[CommandHandler (EditCommands.ToggleAllFoldings)]
+		protected void ToggleAllFoldings ()
+		{
+			GetContent <IFoldable> ().ToggleAllFoldings ();
+		}
+		
+		[CommandHandler (EditCommands.FoldDefinitions)]
+		protected void FoldDefinitions ()
+		{
+			GetContent <IFoldable> ().FoldDefinitions ();
+		}
+		
+		[CommandUpdateHandler (EditCommands.ToggleFolding)]
+		protected void UpdateToggleFolding (CommandInfo info)
+		{
+			info.Enabled = GetContent <IFoldable> () != null;
+		}
+		
+		[CommandHandler (EditCommands.ToggleFolding)]
+		protected void ToggleFolding ()
+		{
+			GetContent <IFoldable> ().ToggleFolding ();
+		}
+		
+		#endregion
+		
+		#region Bookmarks
+		[CommandUpdateHandler (SearchCommands.ToggleBookmark)]
+		[CommandUpdateHandler (SearchCommands.PrevBookmark)]
+		[CommandUpdateHandler (SearchCommands.NextBookmark)]
+		[CommandUpdateHandler (SearchCommands.ClearBookmarks)]
+		protected void UpdateBookmarkCommands (CommandInfo info)
+		{
+			info.Enabled = GetContent <IBookmarkBuffer> () != null;
+		}
+		
+		[CommandHandler (SearchCommands.ToggleBookmark)]
+		public void ToggleBookmark ()
+		{
+			IBookmarkBuffer markBuffer = GetContent <IBookmarkBuffer> ();
+			Debug.Assert (markBuffer != null);
+			int position = markBuffer.CursorPosition;
+			markBuffer.SetBookmarked (position, !markBuffer.IsBookmarked (position));
+		}
+		
+		[CommandHandler (SearchCommands.PrevBookmark)]
+		public void PrevBookmark ()
+		{
+			IBookmarkBuffer markBuffer = GetContent <IBookmarkBuffer> ();
+			Debug.Assert (markBuffer != null);
+			markBuffer.PrevBookmark ();
+		}
+		
+		[CommandHandler (SearchCommands.NextBookmark)]
+		public void NextBookmark ()
+		{
+			IBookmarkBuffer markBuffer = GetContent <IBookmarkBuffer> ();
+			Debug.Assert (markBuffer != null);
+			markBuffer.NextBookmark ();
+		}
+		
+		[CommandHandler (SearchCommands.ClearBookmarks)]
+		public void ClearBookmarks ()
+		{
+			IBookmarkBuffer markBuffer = GetContent <IBookmarkBuffer> ();
+			Debug.Assert (markBuffer != null);
+			markBuffer.ClearBookmarks ();
+		}
+		#endregion
+		
 	}
 }
