@@ -2,7 +2,7 @@
 //     <copyright see="prj:///doc/copyright.txt"/>
 //     <license see="prj:///doc/license.txt"/>
 //     <owner name="Andrea Paatz" email="andrea@icsharpcode.net"/>
-//     <version>$Revision: 2559 $</version>
+//     <version>$Revision: 3125 $</version>
 // </file>
 
 using System;
@@ -48,8 +48,14 @@ namespace ICSharpCode.NRefactory.Parser.VB
 			return curToken;
 		}
 		
+		bool misreadExclamationMarkAsTypeCharacter;
+		
 		protected override Token Next()
 		{
+			if (misreadExclamationMarkAsTypeCharacter) {
+				misreadExclamationMarkAsTypeCharacter = false;
+				return new Token(Tokens.ExclamationMark, Col - 1, Line);
+			}
 			unchecked {
 				int nextChar;
 				while ((nextChar = ReaderRead()) != -1) {
@@ -83,6 +89,7 @@ namespace ICSharpCode.NRefactory.Parser.VB
 						}
 						ch = (char)ReaderRead();
 						
+						bool oldLineEnd = lineEnd;
 						lineEnd = false;
 						while (Char.IsWhiteSpace(ch)) {
 							if (HandleLineEnd(ch)) {
@@ -99,7 +106,7 @@ namespace ICSharpCode.NRefactory.Parser.VB
 						if (!lineEnd) {
 							errors.Error(Line, Col, String.Format("Return expected"));
 						}
-						lineEnd = false;
+						lineEnd = oldLineEnd;
 						continue;
 					}
 					
@@ -148,21 +155,24 @@ namespace ICSharpCode.NRefactory.Parser.VB
 					if (Char.IsLetter(ch)) {
 						int x = Col - 1;
 						int y = Line;
-						string s = ReadIdent(ch);
-						int keyWordToken = Keywords.GetToken(s);
-						if (keyWordToken >= 0) {
-							lineEnd = false;
-							return new Token(keyWordToken, x, y, s);
-						}
-						
-						// handle 'REM' comments
-						if (s.Equals("REM", StringComparison.InvariantCultureIgnoreCase)) {
-							ReadComment();
-							if (!lineEnd) {
-								lineEnd = true;
-								return new Token(Tokens.EOL, Col, Line, "\n");
+						char typeCharacter;
+						string s = ReadIdent(ch, out typeCharacter);
+						if (typeCharacter == '\0') {
+							int keyWordToken = Keywords.GetToken(s);
+							if (keyWordToken >= 0) {
+								// handle 'REM' comments
+								if (keyWordToken == Tokens.Rem) {
+									ReadComment();
+									if (!lineEnd) {
+										lineEnd = true;
+										return new Token(Tokens.EOL, Col, Line, "\n");
+									}
+									continue;
+								}
+								
+								lineEnd = false;
+								return new Token(keyWordToken, x, y, s);
 							}
-							continue;
 						}
 						
 						lineEnd = false;
@@ -225,6 +235,14 @@ namespace ICSharpCode.NRefactory.Parser.VB
 		
 		string ReadIdent(char ch)
 		{
+			char typeCharacter;
+			return ReadIdent(ch, out typeCharacter);
+		}
+		
+		string ReadIdent(char ch, out char typeCharacter)
+		{
+			typeCharacter = '\0';
+			
 			sb.Length = 0;
 			sb.Append(ch);
 			int peek;
@@ -237,7 +255,14 @@ namespace ICSharpCode.NRefactory.Parser.VB
 			}
 			
 			if ("%&@!#$".IndexOf((char)peek) != -1) {
+				typeCharacter = (char)peek;
 				ReaderRead();
+				if (typeCharacter == '!') {
+					peek = ReaderPeek();
+					if (peek != -1 && (peek == '_' || peek == '[' || char.IsLetter((char)peek))) {
+						misreadExclamationMarkAsTypeCharacter = true;
+					}
+				}
 			}
 			return sb.ToString();
 		}
@@ -479,7 +504,7 @@ namespace ICSharpCode.NRefactory.Parser.VB
 			Location start = new Location(Col - 1, Line);
 			string directive = ReadIdent('#');
 			string argument  = ReadToEndOfLine();
-			this.specialTracker.AddPreprocessingDirective(directive, argument.Trim(), start, new Location(start.X + directive.Length + argument.Length, start.Y));
+			this.specialTracker.AddPreprocessingDirective(new PreprocessingDirective(directive, argument.Trim(), start, new Location(start.X + directive.Length + argument.Length, start.Y)));
 		}
 		
 		string ReadDate()
@@ -548,11 +573,11 @@ namespace ICSharpCode.NRefactory.Parser.VB
 				if (missingApostrophes > 0) {
 					if (ch == '\'' || ch == '\u2018' || ch == '\u2019') {
 						if (--missingApostrophes == 0) {
-							specialTracker.StartComment(CommentType.Documentation, startPos);
+							specialTracker.StartComment(CommentType.Documentation, true, startPos);
 							sb.Length = 0;
 						}
 					} else {
-						specialTracker.StartComment(CommentType.SingleLine, startPos);
+						specialTracker.StartComment(CommentType.SingleLine, true, startPos);
 						missingApostrophes = 0;
 					}
 				}
@@ -566,7 +591,7 @@ namespace ICSharpCode.NRefactory.Parser.VB
 						if (specialCommentHash.ContainsKey(tag)) {
 							Location p = new Location(Col, Line);
 							string comment = ch + ReadToEndOfLine();
-							this.TagComments.Add(new TagComment(tag, comment, p, new Location(Col, Line)));
+							this.TagComments.Add(new TagComment(tag, true, comment, p, new Location(Col, Line)));
 							sb.Append(comment);
 							break;
 						}
@@ -574,7 +599,7 @@ namespace ICSharpCode.NRefactory.Parser.VB
 				}
 			}
 			if (missingApostrophes > 0) {
-				specialTracker.StartComment(CommentType.SingleLine, startPos);
+				specialTracker.StartComment(CommentType.SingleLine, true, startPos);
 			}
 			specialTracker.AddString(sb.ToString());
 			specialTracker.FinishComment(new Location(Col, Line));
@@ -710,6 +735,8 @@ namespace ICSharpCode.NRefactory.Parser.VB
 					return new Token(Tokens.CloseCurlyBrace, x, y);
 				case '?':
 					return new Token(Tokens.QuestionMark, x, y);
+				case '!':
+					return new Token(Tokens.ExclamationMark, x, y);
 			}
 			return null;
 		}
