@@ -1,8 +1,8 @@
-﻿// <file>
+// <file>
 //     <copyright see="prj:///doc/copyright.txt"/>
 //     <license see="prj:///doc/license.txt"/>
 //     <owner name="Mike Krüger" email="mike@icsharpcode.net"/>
-//     <version>$Revision: 2517 $</version>
+//     <version>$Revision: 2533 $</version>
 // </file>
 
 using System;
@@ -35,6 +35,7 @@ namespace ICSharpCode.NRefactory
 		void Parse();
 		
 		Expression ParseExpression();
+		TypeReference ParseTypeReference ();
 		BlockStatement ParseBlock();
 		List<INode> ParseTypeMembers();
 	}
